"""
    Implements profile forwarding for users.
"""
from django.core.exceptions import ObjectDoesNotExist

from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL
from slumber.operations import InstanceOperation
from slumber.operations.instancedata import instance_data


class GetProfile(InstanceOperation):
    """Fetches the user proflle from the auth remote server.
    """

    def get(self, _request, response, _appname, _modelname, pk):
        """Implements the profile lookup.
        """
        try:
            user = self.model.model.objects.get(pk=pk)
            profile = user.get_profile()
            instance_data(response,
                DJANGO_MODEL_TO_SLUMBER_MODEL[type(profile)],
                profile)
        except ObjectDoesNotExist:
            response['_meta']['status'] = 404



import copy
from decimal import Decimal
import logging
import mock
import unittest2

from django.http import HttpResponse
import django.test

from slumber import client
from slumber.test import mock_client

from slumber_examples.models import Order
from slumber_examples.tests.views import ServiceTestsWithDirectory


class TestSlumberMock(unittest2.TestCase):
    margarita = dict(pk=1, name='Margarita', for_sale=True)


    @mock_client(app__contrib__auth__Model=[], app__Pizza=[])
    def test_basic_app_data(self):
        """Ensure that the basic meta data part of the mock works as it should.
        """
        self.assertTrue(hasattr(client, 'app'))
        self.assertTrue(hasattr(client.app, 'contrib'))
        self.assertTrue(hasattr(client.app.contrib, 'auth'))
        self.assertTrue(hasattr(client.app.contrib.auth, 'Model'))
        self.assertTrue(hasattr(client.app, 'Pizza'))
        self.assertFalse(hasattr(client, 'slumber'))


    @mock_client(
        slumber__Pizza=[
           margarita ,
            dict(pk=2, name='Four seasons', prices=[
                dict(pk=2, amount=Decimal("13"))
            ]),
            dict(pk=3, name='Hawaiin', for_sale=False),
        ],
        slumber__PizzaPrice= [
            dict(pk=1,
                pizza=margarita,
                date='2010-01-01', amount=Decimal("14")),
        ])
    def test_get_pizza(self):
        """Ensure that we can manage instances and data arrays properly.
        """
        p1 = client.slumber.Pizza.get(pk=1)
        self.assertEquals(p1.pk, 1)
        self.assertEquals(getattr(p1, 'name', None), 'Margarita', p1.__dict__)
        self.assertEquals(type(p1).__name__, "Pizza")

        p2 = client.slumber.Pizza.get(pk=2)
        self.assertEquals(p2.pk, 2)
        self.assertEquals(len(p2.prices), 1)
        self.assertTrue(hasattr(p2.prices[0], 'pk'), type(p2.prices[0]))
        self.assertEquals(p2.prices[0].amount, Decimal("13"))

        pp1 = client.slumber.PizzaPrice.get(pk=1)
        self.assertEquals(pp1.pk, 1)
        self.assertEquals(pp1.pizza.name, 'Margarita')


    @mock_client(django__contrib__auth__User=[])
    def test_not_found_asserts(self):
        with self.assertRaises(AssertionError):
            client.django.contrib.auth.User.get(pk=1)


    @mock_client(app__Model=[
        dict(pk=1)
    ])
    def test_aliase_writes_are_visible(self):
        m1 = client.app.Model.get(pk=1)
        m2 = client.app.Model.get(pk=1)
        self.assertEqual(m1.pk, m2.pk)
        with self.assertRaises(AttributeError):
            m1.attr
        with self.assertRaises(AttributeError):
            m2.attr
        m1.attr = 'attribute data'
        self.assertEqual(m1.attr, 'attribute data')
        self.assertEqual(m1.attr, m2.attr)


class TestMockWithDatabase(ServiceTestsWithDirectory, django.test.TestCase):
    def _setup(self):
        # Can't use setUp as that runs before the mock is applied
        self.shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        self.order = Order(shop=self.shop)
        self.order.save()
        for order in Order.objects.all():
            logging.debug(order.shop)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1, _url='slumber://pizzas/slumber_examples/Shop/data/1/')
    ])
    def test_can_order_with_remote_shop_mock(self):
        self._setup()
        fetched = Order.objects.get(shop=self.shop)
        self.assertEqual(self.order, fetched)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1, _url='slumber://pizzas/slumber_examples/Shop/data/1/')
    ])
    def test_can_order_with_remote_shop_url(self):
        self._setup()
        fetched = Order.objects.get(
            shop='slumber://pizzas/slumber_examples/Shop/data/1/')
        self.assertEqual(self.order, fetched)


class TestViews(django.test.TestCase):
    @mock_client()
    def test_view(self):
        """Make sure that a simple view works with the mocked client.
        """
        self.client.get('/')

    @mock_client()
    @mock.patch('slumber_examples.views.ok_text')
    def test_mocked_then_patched(self, ok_text_patch):
        ok_text_patch.return_value = HttpResponse('patched')
        self.client.get('/')

    @mock.patch('slumber_examples.views.ok_text')
    @mock_client()
    def test_patched_then_mocked(self, ok_text_patch):
        ok_text_patch.return_value = HttpResponse('patched')
        self.client.get('/')


class TestCopy(ServiceTestsWithDirectory, django.test.TestCase):
    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop(self):
        shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        deep = copy.deepcopy(shop)
        self.assertEqual(shop.pk, deep.pk)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query(self):
        query = Order.objects.filter(
            shop='slumber://pizzas/slumber_examples/Shop/data/1/')
        self.assertEqual(query.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_client_shop(self):
        shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        query1 = Order.objects.filter(shop=shop)
        self.assertEqual(query1.count(), 0)
        query2 = query1.exclude(shop=shop)
        self.assertEqual(query2.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_shop_url(self):
        shop = 'slumber://pizzas/slumber_examples/Shop/data/1/'
        query1 = Order.objects.filter(shop=shop)
        self.assertEqual(query1.count(), 0)
        query2 = query1.exclude(shop=shop)
        self.assertEqual(query2.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_shop_from_order(self):
        order = Order.objects.create(
            shop = 'slumber://pizzas/slumber_examples/Shop/data/1/')
        query1 = Order.objects.filter(shop=order.shop)
        self.assertEqual(query1.count(), 1)
        query2 = query1.filter(shop=order.shop)
        self.assertEqual(query2.count(), 1)



from django import forms
from django.contrib.admin.widgets import AdminURLFieldWidget
from django.test import TestCase

from slumber.connector import Client
from slumber.connector.api import get_instance
from slumber.forms import RemoteForeignKeyField

from slumber_examples.models import Order
from slumber_examples.tests.configurations import ConfigureUser
from slumber_examples.tests.views import ServiceTests


class WidgetTest(ConfigureUser, ServiceTests, TestCase):
    class Form(forms.Form):
        rfk = RemoteForeignKeyField(
            model_url='http://localhost:8000/slumber/pizzas/slumber_examples/Pizza/',
            required=True)
    class OptionalForm(forms.Form):
        rfk = RemoteForeignKeyField(
            model_url='http://localhost:8000/slumber/pizzas/slumber_examples/Pizza/',
            required=False)
    class ModelForm(forms.ModelForm):
        class Meta:
            model = Order
    class AdminForm(forms.Form):
        rfk = RemoteForeignKeyField(
            model_url='http://localhost:8000/slumber/pizzas/slumber_examples/Pizza/',
            widget=AdminURLFieldWidget)

    def setUp(self):
        super(WidgetTest, self).setUp()
        self.user.is_superuser = True
        self.user.save()
        self.client = Client()


    def test_default_formfield(self):
        form = WidgetTest.Form()
        self.assertEquals(form.as_p(),
            '''<p><label for="id_rfk">Rfk:</label> '''
                '''<input type="text" name="rfk" id="id_rfk" /></p>''')

    def test_empty_form_submission_with_required_field(self):
        form = WidgetTest.Form(dict(rfk=''))
        self.assertFalse(form.is_valid())
        self.assertEquals(form.as_p(),
            '''<ul class="errorlist"><li>This field is required</li></ul>\n'''
            '''<p><label for="id_rfk">Rfk:</label> '''
                '''<input type="text" name="rfk" id="id_rfk" /></p>''')

    def test_empty_form_submission_with_optional_field(self):
        form = WidgetTest.OptionalForm(dict(rfk=''))
        self.assertTrue(form.is_valid())
        self.assertIsNone(form.cleaned_data['rfk'])

    def test_default_widget_with_data(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        form = WidgetTest.Form(dict(rfk=shop))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_rfk">Rfk:</label> '''
                '''<input type="text" name="rfk" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_rfk" /></p>''')

    def test_default_widget_with_submit_data(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        form = WidgetTest.Form(dict(rfk=shop._url))
        self.assertTrue(form.is_valid())
        self.assertEquals(form.cleaned_data['rfk'].id, shop.id)

    def test_given_bad_url(self):
        form = WidgetTest.Form(dict(rfk=
            'http://localhost:8000/slumber/pizzas/slumber_examples/Shop/2/data/'))
        self.assertFalse(form.is_valid())

    def test_model_form(self):
        form = WidgetTest.ModelForm()
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" id="id_shop" /></p>''')

    def test_model_form_submission_with_object(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        form = WidgetTest.ModelForm(dict(shop=shop))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')
        self.assertEquals(type(form.fields['shop']), RemoteForeignKeyField)
        self.assertTrue(form.is_valid())
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')
        self.assertEquals(form.cleaned_data['shop'].id, shop.id)

    def test_model_form_submission_with_url(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        form = WidgetTest.ModelForm(dict(shop=shop._url))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')
        self.assertEquals(type(form.fields['shop']), RemoteForeignKeyField)
        self.assertTrue(form.is_valid())
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')

    def test_model_form_submission_with_instance_get_instance(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        shop_from_url = get_instance(
            self.client.pizzas.slumber_examples.Shop, shop._url, unicode(shop))
        form = WidgetTest.ModelForm(dict(shop=shop_from_url))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')
        self.assertEquals(type(form.fields['shop']), RemoteForeignKeyField)
        self.assertTrue(form.is_valid())
        self.assertEquals(unicode(form.cleaned_data['shop']), unicode(shop))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')

    def test_model_form_with_order(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        order = Order(shop=shop)
        order.save()
        form = WidgetTest.ModelForm(instance=order)
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')

    def test_model_form_with_order_from_database(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        order = Order(shop=shop)
        order.save()
        order_db = Order.objects.get(pk=order.pk)
        form = WidgetTest.ModelForm(instance=order_db)
        self.assertEquals(form.as_p(),
            '''<p><label for="id_shop">Shop:</label> '''
                '''<input type="text" name="shop" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_shop" /></p>''')

    def test_admin_form_with_order(self):
        shop = self.client.pizzas.slumber_examples.Shop.create(
            name='Shop', slug='shop')
        form = WidgetTest.AdminForm(dict(rfk=shop))
        self.assertEquals(form.as_p(),
            '''<p><label for="id_rfk">Rfk:</label> '''
                '''<input type="text" name="rfk" '''
                    '''value="http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/" '''
                    '''id="id_rfk" /></p>''')



"""
    Implements the conversion of the response data to valid HTTP
    data.
"""
import logging
from simplejson import dumps, JSONEncoder, loads

from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
try:
    from django.views.decorators.csrf import csrf_exempt
    USE_CSRF = True
except ImportError: # pragma: no cover
    USE_CSRF = False

from slumber.server import NotAuthorised, Forbidden


class _proxyEncoder(JSONEncoder):
    """If we don't know how to deal with the attribute type we'll just
    convert to a string and hope that's ok for now.
    """
    # An attribute inherited from JSONEncoder hide this method
    # pylint: disable=E0202
    def default(self, obj):
        return unicode(obj)


def require_user(function):
    """Throw NotAuthorised if the view is accessed anonymously.
    """
    def decorated(cls, request, *args, **kwargs):
        """The docorated function.
        """
        logging.debug("User %s is_authenticated: %s", request.user,
            request.user.is_authenticated())
        if not request.user.is_authenticated():
            raise NotAuthorised()
        return function(cls, request, *args, **kwargs)
    return decorated


def require_permission(permission):
    """Throw Forbidden if the user does not have the required permission
    """
    def decorator(function):
        """The decorator returned from its configuration function.
        """
        @require_user
        def decorated(cls, request, *args, **kwargs):
            """The decorated view function.
            """
            if not request.user.has_perm(permission):
                raise Forbidden("Requires permission %s" % permission)
            return function(cls, request, *args, **kwargs)
        return decorated
    return decorator


def view_handler(view):
    """Wrap a view function so it can return either JSON, HTML or some
    other response.
    """
    def wrapper(request, *args, **kwargs):
        """The decorated implementation.
        """
        meta = request.META
        if meta.get('CONTENT_TYPE', '').startswith('application/json') and \
                meta.get('CONTENT_LENGTH'):
            request.POST = loads(request.raw_post_data)
        response = {'_meta': dict(status=200, message='OK')}
        try:
            http_response = view(request, response, *args, **kwargs)
            if http_response:
                return http_response
        except NotAuthorised, _:
            response = {
                '_meta': dict(status=401, message='Unauthorized',
                    headers= {'WWW-Authenticate':'FOST Realm="Slumber"'}),
                'error': 'No user is logged in'}
        except Forbidden, exception:
            response = {'_meta': dict(status=403, message='Forbidden'),
                'error': unicode(exception)}
        except ObjectDoesNotExist, exception:
            response = {'_meta': dict(status=404, message='Not Found'),
                'error': unicode(exception)}
        except NotImplementedError, _:
            response = {
                '_meta': dict(status=501, message='Not Implemented'),
                'error': "Not implemented"}
        if request.user.is_authenticated():
            response['_meta']['username'] = request.user.username
        else:
            logging.debug("Request user %s not authenticated", request.user)
        http_response = HttpResponse(dumps(response, indent=4,
                cls=_proxyEncoder), 'text/plain',
            status=response['_meta']['status'])
        for header, value in response['_meta'].get('headers', {}).items():
            http_response[header] = value
        return http_response
    return wrapper if not USE_CSRF else csrf_exempt(wrapper)




"""
    Allows the data URL to be found for a given object.
"""
from django.http import HttpResponseNotFound

from slumber.operations import ModelOperation
from slumber.operations.instancedata import instance_data


class DereferenceInstance(ModelOperation):
    """Given a primary key (or other unique set of attributes) redirects
    to the instance item.
    """
    def get(self, request, response, _appname, _modelname):
        """Work out the correct data URL for an instance we're going to
        search for.
        """
        try:
            instance = self.model.model.objects.get(
                **dict([(k, request.GET[k])
                    for k in request.GET.keys()]))
            return instance_data(response, self.model, instance)
        except self.model.model.DoesNotExist:
            return HttpResponseNotFound()



from application_configuration import *
from authentication import *
from client import *
from forms import *
from middleware import *
from models import *
from mock_client import *
from operations import *
from proxies import *
from remote_authn import *
from server import *
from services import *
from ua import *
from views import *



"""
    Implements creation of an object.
"""
from django.core.management.color import no_style
from django.db import connection

from slumber.operations import ModelOperation
from slumber.operations.instancedata import instance_data
from slumber.server.http import require_permission


class CreateInstance(ModelOperation):
    """Allows for the creation of new instances.
    """
    def post(self, request, response, appname, modelname):
        """Perform the object creation.
        """
        @require_permission('%s.add_%s' % (appname, modelname.lower()))
        def do_create(_cls, request):
            """Use an inner function so that we can generate a proper
            permission name at run time.
            """
            instance = self.model.model(**dict([(k, v)
                for k, v in request.POST.items()]))
            instance.save()
            # Reset the sequence point in case there was a PK set
            cursor = connection.cursor()
            lines = connection.ops.sequence_reset_sql(
                no_style(), [self.model.model])
            cursor.execute(';'.join(lines))
            # Return the
            return instance_data(response, self.model, instance)
        return do_create(self, request)




from mock import patch

from django.contrib.auth.models import User
from django.test import TestCase

from slumber import client
from slumber._caches import CLIENT_INSTANCE_CACHE
from slumber.connector import Client, DictObject
from slumber.connector.ua import get

from slumber_examples.models import Pizza, PizzaPrice, PizzaSizePrice
from slumber_examples.tests.configurations import ConfigureUser


class TestDirectoryURLs(ConfigureUser, TestCase):
    def test_get_default_url_with_made_client(self):
        client = Client()
        self.assertEqual('http://localhost:8000/slumber/', client._directory)

    def test_get_default_url_with_default_client(self):
        self.assertEqual('http://localhost:8000/slumber/', client._directory)

    def test_get_default_url_with_services(self):
        with patch('slumber.server._get_slumber_directory', lambda: {
                'pizzas': 'http://localhost:8001/slumber/pizzas/',
                'takeaway': 'http://localhost:8002/slumber/'}):
            client = Client()
        self.assertIsNone(client._directory)


class TestLoads(ConfigureUser, TestCase):

    def test_applications_local(self):
        client = Client('http://localhost:8000/slumber')
        self.assertTrue(hasattr(client, 'slumber_examples'))

    def test_applications_remote(self):
        def request(k, u, headers):
            self.assertEquals(u, 'http://slumber.example.com/')
            return DictObject(status=200), '''{"apps":{}}'''
        with patch('slumber.connector.ua.Http.request', self.fail):
            client = Client('http://slumber.example.com/')
        with patch('slumber.connector.ua.Http.request', request):
            try:
                client.no_module
                self.fail("This should have given an attribute error")
            except AttributeError:
                pass

    def test_applications_with_dots_in_name(self):
        client = Client()
        self.assertTrue(hasattr(client, 'django'), client.__dict__.keys())
        self.assertTrue(hasattr(client.django, 'contrib'), client.django.__dict__.keys())
        self.assertTrue(hasattr(client.django.contrib, 'sites'),
            (type(client.django.contrib), client.django.contrib.__dict__.keys()))
        try:
            client.django.NotAModelOrApp
            self.fail("This should have given an attribute error")
        except AttributeError:
            pass

    def test_new_client_gives_AttributeError_on_invalid_model(self):
        client = Client()
        try:
            client.django.contrib.auth.NotAModelOrApp
            self.fail("This should have given an attribute error")
        except AttributeError:
            pass

    def test_module_attributes(self):
        self.assertTrue(client.slumber_examples.Pizza.module, 'slumber_examples')
        self.assertTrue(client.slumber_examples.Pizza.name, 'Pizza')
        try:
            client.slumber_examples.Pizza.not_a_module_attr
            self.fail("This should have thrown an attribute error")
        except AttributeError:
            pass

    def test_can_create_instance(self):
        self.user.is_superuser = True
        self.user.save()
        rpizza = client.slumber_examples.Pizza.create(
            name='P1', for_sale=True)
        lpizza = Pizza.objects.get(name='P1')
        self.assertEquals(rpizza.id, lpizza.id)


class TestAuth(ConfigureUser, TestCase):
    def test_has_attributes(self):
        user = client.django.contrib.auth.User.get(pk=self.user.pk)
        for attr in ['is_active', 'is_staff', 'date_joined', 'is_superuser',
                'first_name', 'last_name', 'email', 'username']:
            self.assertTrue(hasattr(user, attr), user.__dict__.keys())


class TestsWithPizza(ConfigureUser, TestCase):
    def setUp(self):
        super(TestsWithPizza, self).setUp()
        self.s = Pizza(name='S1', for_sale=True)
        self.s.save()
        self.pizza = client.slumber_examples.Pizza.get(pk=self.s.pk)


    def test_instance_type(self):
        self.assertEqual(self.s.pk, self.pizza.id)
        self.assertEqual(type(self.pizza).__name__,
            'http://localhost:8000/slumber/slumber_examples/Pizza/data/1/')
        pizza_type = str(type(self.pizza))
        self.assertTrue(pizza_type.endswith("slumber_examples/Pizza/data/1/'>"),
            pizza_type)


    def test_cache_ttl(self):
        self.assertEqual(self.pizza._CACHE_TTL, 0)


    def test_instance_data(self):
        self.assertEqual('S1', self.pizza.name)
        prices = self.pizza.prices
        self.assertEqual(len(prices), 0)
        self.assertTrue(self.pizza.exclusive_to is None, self.pizza.exclusive_to)
        try:
            self.pizza.not_a_field
            self.fail("This should have thrown an AttributeError")
        except AttributeError:
            pass


    def test_instance_data_with_data_array(self):
        for p in range(15):
            PizzaPrice(pizza=self.s, date='2011-04-%s' % (p+1)).save()
        self.assertEqual('S1', self.pizza.name)
        prices = self.pizza.prices
        self.assertEquals(len(prices), 15)
        first_price = prices[0]
        self.assertEquals(unicode(first_price), "PizzaPrice object")
        self.assertEquals(first_price.pizza.for_sale, True)


    def test_instance_data_with_nested_data_array(self):
        p = PizzaPrice(pizza=self.s, date='2010-06-20')
        p.save()
        PizzaSizePrice(price=p, size='s', amount='13.95').save()
        PizzaSizePrice(price=p, size='m', amount='15.95').save()
        PizzaSizePrice(price=p, size='l', amount='19.95').save()
        self.assertEqual('S1', self.pizza.name)
        self.assertEqual(len(self.pizza.prices), 1)
        self.assertEqual(len(self.pizza.prices[0].amounts), 3)
        for a in self.pizza.prices[0].amounts:
            self.assertTrue(a.size in ['s', 'm', 'l'], a.size)


    def test_instance_no_pk(self):
        with self.assertRaises(AssertionError):
            pizza = client.slumber_examples.Pizza.get()


    def test_2nd_pizza_comes_from_cache(self):
        # Force a cache read
        self.assertEqual(unicode(self.pizza), u"S1")
        # Make a 2nd alias to the same object
        fail = lambda *a, **f: self.fail("_InstanceConnector.__init__ called again %s, %s" % (a, f))
        with patch('slumber.connector.api._InstanceConnector.__init__', fail):
            pizza2 = client.slumber_examples.Pizza.get(pk=self.s.pk)
            self.assertEqual(unicode(pizza2), u"S1")


    def test_pizza_not_found(self):
        with self.assertRaises(AssertionError):
            p2 = client.slumber_examples.Pizza.get(pk=2)


class AppServiceTests(ConfigureUser, TestCase):
    """Used to get service view tests where the service is configured
    on the application.
    """
    def setUp(self):
        super(AppServiceTests, self).setUp()
        pizzas = lambda: 'pizzas'
        directory = lambda: {
                'auth': 'django.contrib.auth',
                'pizzas': 'http://localhost:8000/slumber/pizzas/',
            }
        self.__patchers = [
            patch('slumber.server._get_slumber_service', pizzas),
            patch('slumber.server._get_slumber_directory', directory),
        ]
        [p.start() for p in self.__patchers]
    def tearDown(self):
        [p.stop() for p in self.__patchers]
        super(AppServiceTests, self).tearDown()

    def test_directory(self):
        request, json = get('http://localhost:8000/slumber/')
        self.assertEquals(json['services'], {
            'auth': 'http://localhost:8000/slumber/pizzas/django/contrib/auth/',
            'pizzas': 'http://localhost:8000/slumber/pizzas/'})

    def test_auth(self):
        self.client = Client()
        self.assertTrue(hasattr(self.client, 'auth'), self.client.__dict__)
        self.assertTrue(hasattr(self.client.auth, 'User'), self.client.auth.__dict__)

    def test_pizzas(self):
        self.client = Client()
        self.assertTrue(hasattr(self.client, 'pizzas'), self.client.__dict__)
        self.assertTrue(hasattr(self.client.pizzas, 'slumber_examples'),
            self.client.pizzas.__dict__)



import logging
from mock import patch
from simplejson import dumps, loads

from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.db import connection
from django.test import TestCase

from slumber import Client
from slumber.connector.ua import _calculate_signature
from slumber.scheme import SlumberServiceURLError
from slumber_examples.models import Order, Pizza, PizzaPrice, Shop
from slumber_examples.tests.configurations import ConfigureUser


def _perform(client, method, url, data, content_type=None):
    def method_wrapper(*a, **kw):
        logging.info("Using wrapped GET handler")
        return client.get(*a, REQUEST_METHOD=method.upper(), **kw)
    logging.info("%s with data %s", method, data)
    headers = _calculate_signature('service', method.upper(), url, data, None, True)
    response = getattr(client, method, method_wrapper)(
        url, data, content_type=content_type, **headers)
    if response.status_code == 200:
        return response, loads(response.content)
    else:
        return response, {}


class ViewTests(object):
    """Base class for view tests that give us some user agent functionality.
    """
    def do_get(self, url, query = {}):
        return _perform(self.client, 'get', self.url(url), query)

    def do_post(self, url, body):
        return _perform(self.client, 'post', self.url(url), dumps(body),
            'application/json')

    def do_options(self, url):
        return _perform(self.client, 'options', self.url(url), {})

    def url(self, path):
        if not path.startswith(self.PREFIX + '/'):
            return self.PREFIX + path
        else:
            return path


class PlainTests(object):
    """Used to get non-service based view tests.
    """
    PREFIX = '/slumber'


class ServiceTests(object):
    """Used to get service based view tests.
    """
    PREFIX  = '/slumber/pizzas'
    def patch(self, what, to):
        patcher = patch(what, to)
        self.__patchers.append(patcher)
        patcher.start()
    def setUp(self):
        self.__patchers = []
        self.patch('slumber.server._get_slumber_service', lambda: 'pizzas')
    def tearDown(self):
        [p.stop() for p in self.__patchers]


class ServiceTestsWithDirectory(ServiceTests):
    def setUp(self):
        super(ServiceTestsWithDirectory, self).setUp()
        directory = lambda: dict(pizzas='http://localhost:8000/slumber/pizzas/')
        self.patch('slumber.server._get_slumber_directory', directory)


class ViewErrors(ViewTests):

    def test_method_error(self):
        response, json = self.do_post('/slumber_examples/Pizza/instances/', {})
        self.assertEquals(response.status_code, 405)
        self.assertEquals(response['Allow'], 'GET, OPTIONS')

    def test_invalid_method(self):
        url = self.url('/slumber_examples/Pizza/instances/')
        response = self.client.get(url, REQUEST_METHOD='PURGE',
            HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        self.assertEquals(response.status_code, 405, response.content)
        self.assertEquals(response['Allow'], 'GET, OPTIONS')

    def test_missing_slash(self):
        response, json = self.do_get('/slumber_examples')
        self.assertEquals(response.status_code, 301)
        self.assertTrue(response['location'].endswith('/slumber_examples/'),
            response['location'])

    def test_invalid_model(self):
        response, json = self.do_get('/slumber_examples/not-a-model/')
        self.assertEquals(response.status_code, 404)

    def test_invalid_model_operation(self):
        response, json = self.do_get('/slumber_examples/Pizza/not-an-operation/')
        self.assertEquals(response.status_code, 404)


class ViewErrorsPlain(ConfigureUser, ViewErrors, PlainTests, TestCase):
    pass
class ViewErrorsService(ConfigureUser, ViewErrors, ServiceTests, TestCase):
    def test_invalid_service(self):
        response = self.client.get('/slumber/not-a-service/')
        self.assertEquals(response.status_code, 404, response.content)


class BasicViews(ViewTests):

    def test_applications(self):
        self.assertTrue(bool(self.user))
        self.assertEqual(self.user.pk, 1)
        response, json = self.do_get('/')
        apps = json['apps']
        self.assertEquals(apps['slumber_examples'], self.url('/slumber_examples/'))

    def test_model_search_success(self):
        response, json = self.do_get('/', {'model': 'slumber_examples.Pizza'})
        self.assertEquals(response.status_code, 302)
        self.assertTrue(response['location'].endswith(
            '/slumber_examples/Pizza/'), response['location'])

    def test_model_search_invalid(self):
        response, json = self.do_get('/', {'model': 'nota.model'})
        self.assertEquals(response.status_code, 404)

    def test_application_with_models(self):
        response, json = self.do_get('/slumber_examples/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(len(json['models']))
        self.assertEquals(json['models']['Pizza'],
            self.url('/slumber_examples/Pizza/'))

    def test_application_without_models(self):
        response, json = self.do_get('/slumber_examples/no_models/')
        self.assertEquals(response.status_code, 200)
        self.assertFalse(len(json['models']))

    def test_nested_application(self):
        response, json = self.do_get('/slumber_examples/nested1/')
        self.assertEquals(response.status_code, 200, 'slumber_examples.nested1')
    def test_doubly_nested_application(self):
        response, json = self.do_get('/slumber_examples/nested1/nested2/')
        self.assertEquals(response.status_code, 200, 'slumber_examples.nested1.nested2')
    def test_models_across_apps(self):
        response, json = self.do_get('/slumber_ex_shop/NestedModel/')
        self.assertEquals(response.status_code, 200, 'slumber_ex_shop.NestedModel')
        response, json = self.do_get('/slumber_ex_shop/Pizza/')
        self.assertEquals(response.status_code, 404, 'slumber_ex_shop.Pizza')

    def test_instance_metadata_pizza(self):
        response, json = self.do_get('/slumber_examples/Pizza/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['fields'].has_key('for_sale'))
        self.assertEquals(json['fields']['for_sale']['type'],
            'django.db.models.fields.BooleanField')
        self.assertEquals(json['operations']['instances'],
            self.url('/slumber_examples/Pizza/instances/'))
        self.assertFalse(json['operations'].has_key('data'), json['operations'])
        self.assertTrue(json['operations'].has_key('get'), json['operations'])

    def test_instance_metadata_pizzaprice(self):
        response, json = self.do_get('/slumber_examples/PizzaPrice/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['fields'].has_key('pizza'))
        self.assertEquals(json['fields']['pizza']['type'],
            self.url('/slumber_examples/Pizza/'))

    def test_model_metadata_user(self):
        response, json = self.do_get('/django/contrib/auth/User/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('authenticate'), json['operations'])
        self.assertEquals(json['operations']['authenticate'],
            self.url('/django/contrib/auth/User/authenticate/'))
        self.assertFalse(json['fields'].has_key('groups'), json['fields'].keys())
        self.assertIn('groups', json['data_arrays'])

    def test_instance_metadata_user(self):
        user = User(username='test-user')
        user.save()
        response, json = self.do_get('/django/contrib/auth/User/data/%s/' %
            user.pk)
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('has-permission'), json['operations'])

    def test_instance_puttable(self):
        response, json = self.do_get('/slumber_examples/Pizza/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['puttable'], [['id'], ['name']])

    def test_model_operation_instances_no_instances(self):
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 0)

    def test_model_operation_instances_one_instance(self):
        Pizza(name='S1', for_sale=True).save()
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 1)

    def test_model_operation_instances_twelve_instances(self):
        for i in range(12):
            Pizza(name='S%s' % i, for_sale=True).save()
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 10)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/instances/?start_after=3'))
        response, json = self.do_get('/slumber_examples/Pizza/instances/',
            {'start_after': '3'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 2)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/instances/?start_after=1'))
        response, json = self.do_get('/slumber_examples/Pizza/instances/',
            {'start_after': '1'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 0)
        self.assertFalse(json.has_key('next_page'), json)

    def test_instance_creation_get(self):
        response, json = self.do_get('/slumber_examples/Pizza/create/')
        self.assertEquals(response.status_code, 405, response.content)
        self.assertEquals(response['Allow'], 'OPTIONS, POST')

    def test_instance_creation_options(self):
        response, json = self.do_options('/slumber_examples/Pizza/create/')
        self.assertEquals(response.status_code, 200, response.content)
        self.assertEquals(response['Allow'], 'OPTIONS, POST')

    def test_instance_creation_post(self):
        self.user.is_superuser = True
        self.user.save()
        response, json = self.do_post('/slumber_examples/Pizza/create/',
            {'name': 'Test Pizza', 'for_sale': ''})
        self.assertTrue(json.get('identity', '').endswith(
            '/slumber_examples/Pizza/data/1/'), json)
        self.assertEquals(Pizza.objects.count(), 1)
        self.assertEquals(Pizza.objects.all()[0].name, 'Test Pizza')
        self.assertFalse(Pizza.objects.all()[0].for_sale)

    def test_update_instance(self):
        self.user.is_superuser = True
        self.user.save()
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_post('/slumber_examples/Pizza/update/1/', {
            'name': 'New pizza'})
        self.assertEquals(response.status_code, 200)
        n = Pizza.objects.get(pk=1)
        self.assertEquals(n.name, "New pizza")

    def test_get_instance(self):
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/')
        get_url = json['operations']['get']
        self.assertEquals(get_url, self.url('/slumber_examples/Pizza/get/'))
        def check_query(query):
            response, json = self.do_get(get_url, query)
            self.assertEquals(response.status_code, 200, response)
            self.assertTrue(json['identity'].endswith(
                '/slumber_examples/Pizza/data/%s/' % s.pk), response)
        check_query({'pk': s.pk})
        check_query({'id': s.pk})
        check_query({'name': s.name})

    def test_instance_data_pizza(self):
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/' % s.pk)
        self.maxDiff = None
        self.assertEquals(json, dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/Pizza/'),
            identity=self.url('/slumber_examples/Pizza/data/1/'),
            display='S1',
            operations=dict(
                data=self.url('/slumber_examples/Pizza/data/1/'),
                delete=self.url('/slumber_examples/Pizza/delete/1/'),
                order=self.url('/slumber_examples/Pizza/order/1/'),
                update=self.url('/slumber_examples/Pizza/update/1/')),
            fields=dict(
                id=dict(data=s.pk, kind='value', type='django.db.models.fields.AutoField'),
                for_sale=dict(data=s.for_sale, kind='value', type='django.db.models.fields.BooleanField'),
                max_extra_toppings=dict(data=s.max_extra_toppings, kind='value', type='django.db.models.fields.IntegerField'),
                name=dict(data=s.name, kind='value', type='django.db.models.fields.CharField'),
                exclusive_to={'data': None, 'kind': 'object', 'type': self.url('/slumber_examples/Shop/')}),
            data_arrays=dict(
                prices=self.url('/slumber_examples/Pizza/data/%s/prices/' % s.pk))))

    def test_instance_data_shop_with_null_active(self):
        s = Shop(name='Test shop', slug='test-shop')
        s.save()
        response, json = self.do_get('/slumber_examples/Shop/data/%s/' % s.pk)
        self.maxDiff = None
        self.assertEquals(json, dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/Shop/'),
            identity=self.url('/slumber_examples/Shop/data/1/'),
            display='Test shop',
            operations=dict(
                data=self.url('/slumber_examples/Shop/data/1/'),
                delete=self.url('/slumber_examples/Shop/delete/1/'),
                update=self.url('/slumber_examples/Shop/update/1/')),
            fields=dict(
                id=dict(data=s.pk, kind='value', type='django.db.models.fields.AutoField'),
                active=dict(data=None, kind='value', type='django.db.models.fields.NullBooleanField'),
                name={'data': 'Test shop', 'kind': 'value', 'type': 'django.db.models.fields.CharField'},
                slug={'data': 'test-shop', 'kind': 'value', 'type': 'django.db.models.fields.CharField'},
                web_address={'data': 'http://www.example.com/test-shop/', 'kind': 'property',
                    'type': 'slumber_examples.Shop.web_address'}),
            data_arrays={'pizza': self.url('/slumber_examples/Shop/data/1/pizza/')}))

    def test_instance_data_pizzaprice(self):
        s = Pizza(name='p1', for_sale=True)
        s.save()
        p = PizzaPrice(pizza=s, date='2010-01-01')
        p.save()
        response, json = self.do_get('/slumber_examples/PizzaPrice/data/%s/' % p.pk)
        self.assertEquals(json, dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/PizzaPrice/'),
            identity=self.url('/slumber_examples/PizzaPrice/data/1/'),
            display="PizzaPrice object",
            operations=dict(
                data=self.url('/slumber_examples/PizzaPrice/data/1/'),
                delete=self.url('/slumber_examples/PizzaPrice/delete/1/'),
                update=self.url('/slumber_examples/PizzaPrice/update/1/')),
            fields=dict(
                id={'data': 1, 'kind': 'value', 'type': 'django.db.models.fields.AutoField'},
                pizza={'data': {
                        'type': self.url('/slumber_examples/Pizza/'), 'display':'p1',
                        'data': self.url('/slumber_examples/Pizza/data/1/')},
                    'kind': 'object', 'type': self.url('/slumber_examples/Pizza/')},
                date={'data': '2010-01-01', 'kind': 'value', 'type': 'django.db.models.fields.DateField'},
            ),
            data_arrays={'amounts': self.url('/slumber_examples/PizzaPrice/data/1/amounts/')}))

    def test_instance_data_array(self):
        s = Pizza(name='P', for_sale=True)
        s.save()
        for p in range(15):
            PizzaPrice(pizza=s, date='2011-04-%s' % (p+1)).save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/prices/' % s.pk)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 10, json)
        self.assertTrue(json.has_key('next_page'), json)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/data/1/prices/?start_after=6'),
            json['next_page'])
        response, json = self.do_get('/slumber_examples/Pizza/data/1/prices/',
            {'start_after': '6'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 5)
        self.assertEquals(json['page'][0], {
            'type': self.url('/slumber_examples/PizzaPrice/'),
            'pk': 5, 'data': self.url('/slumber_examples/PizzaPrice/data/5/'), 'display': 'PizzaPrice object'})
        self.assertFalse(json.has_key('next_page'), json.keys())

    def test_delete_instance(self):
        self.user.is_superuser = True
        self.user.save()
        s = Pizza(name='P')
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/' % s.pk)
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('delete'), json['operations'])
        response, json = self.do_post(json['operations']['delete'], {})
        self.assertEquals(response.status_code, 200)
        with self.assertRaises(Pizza.DoesNotExist):
            Pizza.objects.get(pk=s.pk)


class BasicViewsPlain(ConfigureUser, BasicViews, PlainTests, TestCase):
    def test_service_configuration_missing_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        self.assertIsNotNone(order.shop)
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0], order.shop._url)
        with self.assertRaises(SlumberServiceURLError):
            order2 = Order.objects.get(pk=order.pk)


class BasicViewsService(ConfigureUser, BasicViews, ServiceTests, TestCase):
    def test_services_with_directory(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': 'http://localhost:8000:/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            response = self.client.get('/slumber/',
                HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        json = loads(response.content)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(json['services'].get('pizzas', None),
            'http://localhost:8000:/slumber/pizzas/', json)
        self.assertEqual(json['services'].get('takeaway', None),
            'http://localhost:8002:/slumber/', json)

    def test_services_without_directory(self):
        response = self.client.get('/slumber/',
            HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        json = loads(response.content)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertTrue(json['services'].get('pizzas', '').endswith('/slumber/pizzas/'), json)

    def test_service_configuration_works_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.pizzas.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        self.assertIsNotNone(order.shop)
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0], 'slumber://pizzas/slumber_examples/Shop/data/1/')
        order2 = Order.objects.get(pk=order.pk)
        self.assertEquals(unicode(order2.shop), unicode(order.shop))
        self.assertEquals(order2.shop.id, order.shop.id)


class BasicViewsWithServiceDirectory(ConfigureUser, BasicViews,
        ServiceTestsWithDirectory, TestCase):
    def test_service_configuration_works_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.pizzas.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0],
            'slumber://pizzas/slumber_examples/Shop/data/%s/' % shop.id)
        order2 = Order.objects.get(pk=order.pk)
        self.assertEquals(unicode(order2.shop), unicode(order.shop))
        self.assertEquals(order2.shop.id, order.shop.id)


class UserViews(ViewTests):
    authn = '/django/contrib/auth/User/authenticate/'
    data = '/django/contrib/auth/User/data/%s/'
    perm = '/django/contrib/auth/User/has-permission/%s/%s/'
    perms = '/django/contrib/auth/User/get-permissions/%s/'

    def setUp(self):
        self.user = User(username='test-user')
        self.user.set_password('password')
        self.user.save()
        super(UserViews, self).setUp()

    def test_user_data(self):
        response, json = self.do_get(self.data % self.user.pk)
        self.assertEqual(response.status_code, 200)
        self.assertIn('is_superuser', json['fields'].keys())
        self.assertIn('date_joined', json['fields'].keys())

    def test_user_not_found(self):
        response, json = self.do_post(self.authn, dict(username='not-a-user', password=''))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], False, json)
        self.assertIsNone(json['user'], json)

    def test_user_wrong_password(self):
        response, json = self.do_post(self.authn,
            dict(username=self.user.username, password='wrong'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], False, json)
        self.assertIsNone(json['user'], json)

    def test_user_authenticates(self):
        response, json = self.do_post(self.authn,
            dict(username=self.user.username, password='password'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], True, json)
        self.assertDictContainsSubset(
            {'pk': self.user.pk, 'display_name': 'test-user'},
            json['user'])
        self.assertTrue(
            json['user']['url'].endswith('/django/contrib/auth/User/data/3/'),
            json['user']['url'])

    def test_user_permission_no_permission(self):
        response, json = self.do_get(self.perm % (self.user.pk, 'foo.example'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], False, json)

    def test_user_permission_is_allowed(self):
        permission = Permission(content_type_id=1, name='Can something',
            codename='can_something')
        permission.save()
        self.user.user_permissions.add(permission)
        response, json = self.do_get(self.perm % (self.user.pk, 'auth.can_something'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], True, json)

    def test_user_permission_not_allowed(self):
        permission = Permission(content_type_id=1, name='Can something',
            codename='can_something')
        permission.save()
        response, json = self.do_get(self.perm % (self.user.pk, 'auth.can_something'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], False, json)

    def test_get_group_permissions(self):
        response, json = self.do_get(self.perms % self.user.pk)
        self.assertEquals(response.status_code, 200)
        self.assertItemsEqual(json['group_permissions'], [])


class UserViewsPlain(ConfigureUser, UserViews, PlainTests, TestCase):
    pass
class UserViewsService(ConfigureUser, UserViews, ServiceTests, TestCase):
    pass




"""
    Middleware to help manage the Slumber client.
"""
from slumber import client
from slumber._caches import CLIENT_INSTANCE_CACHE, PER_THREAD


# Django defines the class members as methods
# pylint: disable=R0201


class Cache(object):
    """This middleware flushes the Slumber client cache at the start
    of each request.
    """

    def process_request(self, _request):
        """Turn the cache on.
        """
        CLIENT_INSTANCE_CACHE.enabled = True

    def process_response(self, _request, response):
        """Turn the cache off again at the end of the request and flush it.
        """
        CLIENT_INSTANCE_CACHE.enabled = False
        client._flush_client_instance_cache()
        return response


class ForwardAuthentication(object):
    """Used to forward authentication of the currently logged in user to
    another backend.
    """

    def process_request(self, request):
        """Save the request in thread local storage so it can be retrieved
        by the user agent when it makes requests.
        """
        if request.user.is_authenticated():
            PER_THREAD.username = request.user.username

    def process_response(self, _request, response):
        """Forget the request, but do an assert to make sure nothing horrible
        has happened to it first.
        """
        PER_THREAD.username = None
        return response



from slumber import configure


class ShopProxy(object):
    def has_shop_proxy(self):
        return True
configure('/slumber_examples/Shop/',
    model_proxy = ShopProxy)


class PizzaProxy(object):
    def has_pizza_proxy(self):
        return True
configure('/slumber_examples/Pizza/',
    instance_proxy = PizzaProxy)



from datetime import datetime
import logging
from mock import patch
from simplejson import loads

from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse
from django.test import TestCase

from fost_authn.authentication import FostBackend

from slumber import client
from slumber._caches import PER_THREAD
from slumber.connector.authentication import Backend, \
    ImproperlyConfigured
from slumber.connector.ua import  _sign_request, get
from slumber.test import mock_client
from slumber_examples.models import Pizza, Profile
from slumber_examples.tests.configurations import ConfigureUser, \
    ConfigureAuthnBackend, PatchForAuthnService


class TestAuthnRequired(ConfigureUser, TestCase):
    def setUp(self):
        self.pizza = Pizza(name='Before change')
        self.pizza.save()
        super(TestAuthnRequired, self).setUp()

    def test_directory_works_when_not_authenticated(self):
        response = self.client.get('/slumber/')
        self.assertEqual(response.status_code, 200)

    def test_model_requires_authentication(self):
        response = self.client.get('/slumber/slumber_examples/Pizza/data/%s/' % self.pizza.pk)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.get('WWW-Authenticate', None),
            'FOST Realm="Slumber"')

    def test_authenticated(self):
        response = self.signed_get(None,
            '/slumber/slumber_examples/Pizza/data/234234/')
        self.assertEqual(response.status_code, 404)
        json = loads(response.content)
        self.assertEqual(json["error"], "Pizza matching query does not exist.")

    def test_model_create_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/create/',
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 403)

    def test_model_create_checks_correct_permission(self):
        permission = Permission.objects.get(codename="add_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(None,
            '/slumber/slumber_examples/Pizza/create/',
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 200, response.content)

    def test_model_update_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/update/%s/' % self.pizza.pk,
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 403)

    def test_model_update_checks_correct_permission(self):
        permission = Permission.objects.get(codename="change_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/update/%s/' % self.pizza.pk,
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 200, response.content)

    def test_model_delete_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/delete/%s/' % self.pizza.pk, {})
        self.assertEqual(response.status_code, 403)

    def test_model_delete_checks_correct_permission(self):
        permission = Permission.objects.get(codename="delete_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/delete/%s/' % self.pizza.pk, {})
        self.assertEqual(response.status_code, 200, response.content)


class TestAuthnForwarding(ConfigureUser, TestCase):
    def setUp(self):
        settings.MIDDLEWARE_CLASSES.append(
            'slumber.connector.middleware.ForwardAuthentication')
        super(TestAuthnForwarding, self).setUp()
    def tearDown(self):
        super(TestAuthnForwarding, self).tearDown()
        settings.MIDDLEWARE_CLASSES.remove(
            'slumber.connector.middleware.ForwardAuthentication')

    def test_signing_function_signs(self):
        headers = {}
        def check_request(request):
            for k, v in _sign_request('GET', '/', '', False).items():
                headers[k] = v
            return HttpResponse('ok', 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.client.get('/', REMOTE_ADDR='127.0.0.1')
        self.assertTrue(headers.has_key('Authorization'), headers)

    def test_username_with_colon(self):
        self.user.username = "my:name"
        self.user.save()
        headers = {}
        def check_request(request):
            for k, v in _sign_request('GET', '/', '', False).items():
                headers[k] = v
            return HttpResponse('ok', 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.signed_get(self.user.username)
        self.assertTrue(headers.has_key('Authorization'), headers)
        self.assertTrue(headers.has_key('X-FOST-User'), headers)
        self.assertEqual(headers['X-FOST-User'], self.user.username)

    def test_authentication_backend_accepts_signature(self):
        def check_request(request):
            class response:
                status = 200
                content = '''null'''
            def _request(_self, url, headers={}):
                backend = FostBackend()
                authz = headers['Authorization']
                key = authz[5:5+len(self.service.username)]
                signature = authz[6+len(self.service.username):]
                logging.info('Authorization %s %s', key, signature)
                request.META['HTTP_X_FOST_TIMESTAMP'] = headers[
                    'X-FOST-Timestamp']
                request.META['HTTP_X_FOST_HEADERS'] = headers[
                    'X-FOST-Headers']
                user = backend.authenticate(request=request,
                    key=key, hmac=signature)
                self.assertTrue(user)
                r = response()
                return r, r.content
            with patch('slumber.connector.ua.Http.request', _request):
                response, json = get('http://example.com/')
            return HttpResponse(response.content, 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.signed_get(self.user.username)


class TestBackend(PatchForAuthnService, TestCase):
    def setUp(self):
        super(TestBackend, self).setUp()
        self.backend = Backend()

    def test_remote_user(self):
        user = client.auth.django.contrib.auth.User.get(username='user')
        for attr in ['is_active', 'is_staff', 'date_joined', 'is_superuser',
                'first_name', 'last_name', 'email', 'username']:
            self.assertTrue(hasattr(user, attr), user.__dict__.keys())

    def test_remote_login(self):
        user = self.backend.authenticate(username=self.user.username, password='pass')
        self.assertTrue(user)
        self.assertEqual(user.username, self.user.username)

    def test_remote_login_with_wrong_password(self):
        user = self.backend.authenticate(username=self.user.username, password='xxxx')
        self.assertIsNone(user)

    def test_login_type_not_recognised(self):
        user = self.backend.authenticate(made_up=True)
        self.assertIsNone(user)

    def test_get_user(self):
        user = self.backend.get_user(self.user.username)
        self.assertEqual(user.username, self.user.username)
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertFalse(user.is_superuser)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertEqual(user.username, user.remote_user.username)
        self.assertEqual(user.is_active, user.remote_user.is_active)
        self.assertEqual(user.is_staff, user.remote_user.is_staff)
        self.assertEqual(user.is_superuser, user.remote_user.is_superuser)

    def test_get_user_with_id(self):
        user = self.backend.get_user(self.user.pk)
        self.assertTrue(user)

    def test_cache_ttl(self):
        user = self.backend.get_user(self.user.username)
        self.assertEqual(user.remote_user._CACHE_TTL, 120)

    def test_group_permissions(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        perms = self.backend.get_group_permissions(user)
        self.assertEqual(perms, self.user.get_group_permissions())

    def test_all_permissions(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        perms = self.backend.get_all_permissions(user)
        self.assertEqual(perms, self.user.get_all_permissions())

    def test_module_perms(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_module_perms(user, 'slumber_examples'))

    def test_existing_permission(self):
        self.assertTrue(bool(ContentType.objects.all().count()))
        content_type = ContentType.objects.get(
            app_label='slumber_examples', model='pizza')
        permission = Permission.objects.get(
            codename='add_pizza', content_type=content_type)
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'slumber_examples.add_pizza'))

    def test_existing_permission(self):
        self.assertTrue(bool(ContentType.objects.all().count()))
        content_type = ContentType.objects.get(
            app_label='slumber_examples', model='pizza')
        permission = Permission.objects.get(
            codename='add_pizza', content_type=content_type)
        self.user.user_permissions.add(permission)
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertTrue(self.backend.has_perm(user, 'slumber_examples.add_pizza'))

    def test_missing_permission(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'slumber_examples.not-a-perm'))
        perm = Permission.objects.get(codename='not-a-perm',
            content_type__app_label='slumber_examples')
        self.assertEqual(perm.codename, 'not-a-perm')
        self.assertEqual(perm.name, perm.codename)
        self.assertEqual(perm.content_type.app_label, 'slumber_examples')
        self.assertEqual(perm.content_type.model, 'unknown')

    def test_permission_with_new_app(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'not-an-app.not-a-perm'))
        perm = Permission.objects.get(codename='not-a-perm',
            content_type__app_label='not-an-app')
        self.assertEqual(perm.codename, 'not-a-perm')
        self.assertEqual(perm.name, perm.codename)
        self.assertEqual(perm.content_type.app_label, 'not-an-app')
        self.assertEqual(perm.content_type.model, 'unknown')

    def test_permission_with_invalid_name(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'not-a-perm'))
        self.assertFalse(self.backend.has_perm(user, 'not-an-app..not-a-perm'))

    def test_user_profile_when_no_profile(self):
        user = self.backend.get_user(self.user.username)
        with self.assertRaises(AssertionError):
            profile = user.get_profile()

    def test_user_profile_when_there_is_a_profile(self):
        profile = Profile(user=self.user)
        profile.save()
        user = self.backend.get_user(self.user.username)
        remote_profile = user.get_profile()
        self.assertEqual(remote_profile.id, profile.id)
        self.assertEqual(remote_profile.user.id, self.user.id)


class AuthenticationTests(ConfigureAuthnBackend, TestCase):
    def save_user(self, request):
        self.user = request.user
        return HttpResponse('ok')

    @mock_client(
        django__contrib__auth__User = []
    )
    def test_improperly_configured(self):
        with self.assertRaises(ImproperlyConfigured):
            self.signed_get('testuser')

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='testuser', is_active=True, is_staff=True,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_is_authenticated(self):
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get('testuser')
        self.assertTrue(self.user.is_authenticated())
        self.assertEqual(self.user.username, 'testuser')

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='testuser', is_active=True, is_staff=False,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_created_user_sees_changes(self):
        self.signed_get('testuser')
        remote_user = client.auth.django.contrib.auth.User.get(
            username='testuser')
        remote_user.is_staff = True
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get('testuser')
        self.assertTrue(self.user.is_staff)

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='admin', is_active=True, is_staff=True,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_admin_is_authenticated(self):
        admin = User(username='admin')
        admin.save()
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get(admin.username)
        self.assertTrue(self.user.is_authenticated())
        self.assertEqual(admin, self.user)

    @mock_client(
        auth__django__contrib__auth__User = []
    )
    def test_remote_user_not_found(self):
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.client.get('/', HTTP_X_FOST_USER='testuser')
        self.assertFalse(self.user.is_authenticated())



"""
    Integration between Slumber and Django models.
"""
from django.db.models import URLField, SubfieldBase

from slumber.connector.api import _InstanceProxy, get_instance
from slumber.connector.dictobject import DictObject
from slumber.forms import RemoteForeignKeyField
from slumber.scheme import to_slumber_scheme, from_slumber_scheme
from slumber.server import get_slumber_services


class RemoteForeignKey(URLField):
    """Wraps Django's URLField to provide a field that references a remote
    object on another Slumber service.
    """
    # Django already has too many public methods and we can't change it
    # pylint: disable=R0904

    description = "A remote Slumber object."
    __metaclass__ = SubfieldBase

    def __init__(self, model_url, **kwargs):
        self.model_url = model_url
        super(RemoteForeignKey, self).__init__(**kwargs)

    def run_validators(self, value):
        # Do not rely on validators as we want to support Django 1.0
        pass

    def get_db_prep_value(self, value, *a, **kw):
        if isinstance(value, basestring):
            return value
        url = to_slumber_scheme(value._url, get_slumber_services())
        return super(RemoteForeignKey, self).get_db_prep_value(url, *a, **kw)

    def get_prep_value(self, value, *a, **kw):
        if isinstance(value, basestring) or isinstance(value, DictObject):
            return value
        url = to_slumber_scheme(value._url, get_slumber_services())
        return super(RemoteForeignKey, self).get_prep_value(url, *a, **kw)

    def to_python(self, value):
        if not value:
            return None
        if isinstance(value, _InstanceProxy):
            return value
        instance_url = from_slumber_scheme(
            super(RemoteForeignKey, self).to_python(value),
            get_slumber_services())
        model_url = from_slumber_scheme(
            self.model_url, get_slumber_services())
        return get_instance(model_url, instance_url, None)

    def formfield(self, **kwargs):
        defaults = {'form_class': RemoteForeignKeyField,
            'model_url': self.model_url}
        defaults.update(kwargs)
        return super(RemoteForeignKey, self).formfield(**defaults)



"""
    Contains widgets used for Slumber.
"""
from django import forms
from django.contrib.admin.widgets import AdminURLFieldWidget

from slumber.connector.api import _InstanceProxy, get_instance
from slumber.scheme import from_slumber_scheme
from slumber.server import get_slumber_services


class RemoteForeignKeyWidget(forms.TextInput):
    """A widget that allows the URL to be edited.
    """
    def render(self, name, value, **kw):
        if isinstance(value, basestring):
            return super(RemoteForeignKeyWidget, self).render(
                name, value, **kw)
        else:
            return super(RemoteForeignKeyWidget, self).render(
                name, value._url if value else None, **kw)


class RemoteForeignKeyField(forms.Field):
    """A simple widget that allows the URL for the remote object to be
    seen and edited.
    """
    def __init__(self, max_length=None, verify_exists=True,
            model_url=None, **kwargs):
        assert model_url, "RemoteForiegnKeyField must be passed a model_url"
        self.max_length = max_length
        self.model_url = model_url
        self.verify_exists = verify_exists
        default = {'widget': RemoteForeignKeyWidget}
        default.update(kwargs)
        if default['widget'] == AdminURLFieldWidget:
            # We have to ignore a request for admin's broken widget
            default['widget'] = RemoteForeignKeyWidget
        super(RemoteForeignKeyField, self).__init__(**default)

    def clean(self, value):
        if not value:
            if self.required:
                raise forms.ValidationError('This field is required')
            return None
        elif isinstance(value, _InstanceProxy):
            return value
        else:
            try:
                model_url = from_slumber_scheme(
                    self.model_url, get_slumber_services())
                instance = get_instance(model_url, value, None)
                unicode(instance)
            except AssertionError:
                raise forms.ValidationError("The remote object doesn't exist")
            return instance



from slumber.operations import InstanceOperation


class OrderPizza(InstanceOperation):
    def get(self, request, response, app, model, pk):
        response['form'] = dict(
            quantity='integer')

    def post(self, _request, _response, _app, _model, _pk):
        raise NotImplementedError("OrderPizza.post")



"""
    Implements the server side operations on models and instances.
"""
from django.core.urlresolvers import reverse


def _forbidden(_request, response, *_):
    """Return an error to say that the method type is not allowed.
    """
    response['_meta']['status'] = 405
    response['_meta']['message'] = "Method Not Allowed"


class ModelOperation(object):
    """Base class for model operations.
    """
    METHODS = ['GET', 'OPTIONS', 'POST', 'PUT', 'DELETE']
    model_operation = True
    def __init__(self, model, name):
        self.model = model
        self.name = name
        self.regex = ''
        self.path = model.path + name + '/'

    def headers(self, retvalue, request, response):
        """Calculate and place extra headers needed for certain types of
        response.
        """
        if response['_meta']['status'] == 405 or request.method == 'OPTIONS':
            response['_meta'].setdefault('headers', {})
            response['_meta']['headers']['Allow'] = \
                ', '.join([method
                    for method in self.METHODS
                        if hasattr(self, method.lower())])
        return retvalue


    def operation(self, request, response, *args):
        """Perform the requested operation in the server.
        """
        if request.method in self.METHODS:
            retvalue = getattr(self, request.method.lower(), _forbidden)(
                request, response, *args)
            return self.headers(retvalue, request, response)
        else:
            _forbidden(request, response)
        return self.headers(None, request, response)


    def options(self, request, response, *_):
        """A standard options response that will fill in the Allow header.
        """
        return self.headers(None, request, response)


class InstanceOperation(ModelOperation):
    """Base class for operations on instances.
    """
    model_operation = False
    def __init__(self, model, name):
        super(InstanceOperation, self).__init__(model, name)
        self.regex = '([^/]+)/'



"""
    Allow us to get an instance directly from the JSON data for an object.
"""
from urllib import urlencode
from urlparse import urljoin, urlparse

from slumber._caches import CLIENT_INSTANCE_CACHE, \
    MODEL_URL_TO_SLUMBER_MODEL
from slumber.connector.configuration import INSTANCE_PROXIES, MODEL_PROXIES
from slumber.connector.dictobject import DictObject
from slumber.connector.json import from_json_data
from slumber.connector.ua import get, post


def _ensure_absolute(url):
    """Assert that a given URL is absolute.
    """
    assert urlparse(url)[0], "The URL <%s> must be absolute" % url


def get_instance(model, instance_url, display_name, fields = None):
    """Return an instance of the specified model etc.
    """
    if isinstance(model, basestring):
        model = get_model(model) # Assume we've been given a URL
    fields = fields or {}
    bases = [_InstanceProxy]
    for type_url, proxy in INSTANCE_PROXIES.items():
        if model._url.endswith(type_url):
            bases.append(proxy)
    type_name = str(instance_url)
    instance_type = type(type_name, tuple(bases), {})
    return instance_type(instance_url, display_name, fields)


def get_model(url):
    """Return the client model connector for a given URL.
    """
    if not MODEL_URL_TO_SLUMBER_MODEL.has_key(url):
        bases = [ModelConnector]
        for type_url, proxy in MODEL_PROXIES.items():
            if url.endswith(type_url):
                bases.append(proxy)
        model_type = type(str(url), tuple(bases), {})
        return model_type(url)
    else:
        return MODEL_URL_TO_SLUMBER_MODEL[url]


#def get_instance_from_url(url):
    #"""Returns a local instance proxy for the object described by the
    #absolute URL provided.
    #"""
    ## This is a bit ugly, but we need to do proper proxy lookups for real use
    ## and the mock is a pain to do anything else with
    #from slumber import _client
    #if isinstance(_client, _MockClient):
        #return _InstanceProxy(url, None)
    #else:
        #_, json = get(url)
        #return get_instance_from_data(url, json)


def get_instance_from_data(base_url, json):
    """Return a local instance proxy for the object described by the provided
    JSON like Python datastructure.
    """
    model = get_model(urljoin(base_url, json['type']))
    instance_url = urljoin(base_url, json['operations']['data'])
    return get_instance(model, instance_url, json['display'],
        dict([(k, from_json_data(base_url, j))
            for k, j in json['fields'].items()]))


class ModelConnector(DictObject):
    """Handles the connection to a Django model.
    """
    _CACHE_TTL = 2

    def __init__(self, url, **kwargs):
        _ensure_absolute(url)
        assert not MODEL_URL_TO_SLUMBER_MODEL.has_key(url), \
            (url, MODEL_URL_TO_SLUMBER_MODEL.keys())
        MODEL_URL_TO_SLUMBER_MODEL[url] = self
        self._url = url
        super(ModelConnector, self).__init__(**kwargs)

    def __call__(self, url, display_name):
        """Construct an instance of this model.
        """
        return get_instance(self, url, display_name)

    def __getattr__(self, name):
        attrs = ['name', 'module']
        if name in attrs + ['_operations']:
            _, json = get(self._url, self._CACHE_TTL)
            # We need to set this outside of __init__ for it to work correctly
            # pylint: disable = W0201
            self._operations = dict([(o, urljoin(self._url, u))
                for o, u in json['operations'].items()])
            for attr in attrs:
                setattr(self, attr, json[attr])
            return getattr(self, name)
        else:
            raise AttributeError(name)

    def create(self, **kwargs):
        """Implements the client side for the model `create` operator.
        """
        url = urljoin(self._url, self._operations['create'])
        _, json = post(url, kwargs)
        return get_instance_from_data(url, json)

    def get(self, **kwargs):
        """Implements the client side for the model 'get' operator.
        """
        assert len(kwargs), \
            "You must supply kwargs to filter on to fetch the instance"
        url = urljoin(self._url, 'get/')
        _, json = get(url + '?' + urlencode(kwargs), self._CACHE_TTL)
        return get_instance_from_data(url, json)


class _InstanceProxy(object):
    """Add an extra layer of indirection between the objects being manipulated
    by the application code and the underlying object. This allows us to
    better handle the cache.
    """
    def __init__(self, url, display, fields = None):
        super(_InstanceProxy, self).__init__()
        self._url = url
        self._display = display
        self._fields = fields or {}

    def __deepcopy__(self, _memo):
        """Return a deep copy of the proxy. This isn't really deep.
        """
        return type(self)(self._url, self._display, self._fields)

    def _fetch_instance(self):
        """Fetch the underlying instance.
        """
        instance = CLIENT_INSTANCE_CACHE.get(self._url, None)
        if not instance:
            # We now have a cache miss so construct a new connector
            instance = _InstanceConnector(self._url, **self._fields)
            if CLIENT_INSTANCE_CACHE.enabled:
                CLIENT_INSTANCE_CACHE[self._url] = instance
        return instance

    def __getattr__(self, name):
        """Fetch the underlying instance from the cache if necessary and
        return the attribute value it has.
        """
        if name in ['_as_sql', 'as_sql', 'evaluate', 'get_compiler',
                'get_placeholder', '__iter__', 'next', '_prepare', 'prepare',
                'prepare_database_save', 'value_annotation']:
            # These are attributes that we will never have
            raise AttributeError(name)
        return getattr(self._fetch_instance(), name)

    def __setattr__(self, name, value):
        """Write the attribute through to the underlying instance.
        """
        if name.startswith('_'):
            return super(_InstanceProxy, self).__setattr__(name, value)
        return setattr(self._fetch_instance(), name, value)

    def __unicode__(self):
        """Allow us to take the unicode name of the instance
        """
        if not self._display:
            instance = self._fetch_instance()
            self._display = instance._display
        return self._display


def _return_data_array(base_url, arrays, instance, name, cache_ttl):
    """Implement the lazy fetching of the instance data.
    """
    # Pylint makes a bad type deduction
    # pylint: disable=E1103
    if name in arrays.keys():
        data_array = []
        _, data = get(urljoin(base_url, arrays[name]), cache_ttl)
        while True:
            for obj in data['page']:
                model_url = urljoin(base_url, obj['type'])
                model = MODEL_URL_TO_SLUMBER_MODEL[model_url]
                instance_url = urljoin(base_url, obj['data'])
                data_array.append(
                    get_instance(model, instance_url, obj['display']))
            if data.has_key('next_page'):
                _, data = get(urljoin(base_url, data['next_page']), cache_ttl)
            else:
                break
        setattr(instance, name, data_array)
        return data_array
    else:
        raise AttributeError(name)


class _InstanceConnector(DictObject):
    """Connects to a remote instance.
    """
    _CACHE_TTL = 0

    def __init__(self, url, **kwargs):
        self._url = url
        super(_InstanceConnector, self).__init__(**kwargs)

    def _fetch_data(self):
        """Force fetching the data for this instance.
        """
        _, json = get(self._url, self._CACHE_TTL)
        # We need to set this outside of __init__ for it to work correctly
        # pylint: disable = W0201
        self._operations = dict([(o, urljoin(self._url, u))
            for o, u in json['operations'].items()])
        for k, v in json['fields'].items():
            setattr(self, k, from_json_data(self._url, v))
        self._display = json['display']
        return json

    def __getattr__(self, name):
        json = self._fetch_data()
        if name in json['fields'].keys() + ['_operations', '_display']:
            return getattr(self, name)
        else:
            return _return_data_array(
                self._url, json['data_arrays'], self, name, self._CACHE_TTL)


# This is at the end to ensure that the built in proxies are loaded up properly
# We also don't care that the import is unused
# pylint: disable = W0611
import slumber.connector.proxies




"""
    Implements a listing of all instances for a given model.
"""
from slumber.operations import ModelOperation
from slumber.server import get_slumber_root


class InstanceList(ModelOperation):
    """Allows access to the instances.
    """
    def get(self, request, response, _appname, _modelname):
        """Return a paged set of instances for this model.
        """
        root = get_slumber_root()
        response['model'] = root + self.model.path

        query = self.model.model.objects.order_by('-pk')
        if request.GET.has_key('start_after'):
            query = query.filter(pk__lt=request.GET['start_after'])

        response['page'] = [
                dict(pk=o.pk, display=unicode(o),
                    data=root + self.model.path + 'data/%s/' % o.pk)
            for o in query[:10]]
        if len(response['page']) > 0:
            response['next_page'] = root +self.model.path + \
                'instances/?start_after=%s' % response['page'][-1]['pk']



"""
    Implements authorization checks for users.
"""
from slumber.operations import InstanceOperation


class PermissionCheck(InstanceOperation):
    """Allows for checking if a given user has a specific permission.
    """

    def get(self, _request, response, _appname, _modelname, pk, permission):
        """Implements the permission lookup.
        """
        user = self.model.model.objects.get(pk=pk)
        response['is-allowed'] = user.has_perm(permission)


class ModulePermissions(InstanceOperation):
    """Allows us to check if the user has any permissions for the module.
    """

    def get(self, _request, response, _appname, _modelname, pk, module):
        """Implements the wrapper.
        """
        user = self.model.model.objects.get(pk=pk)
        response['has_module_perms'] = user.has_module_perms(module)


class GetPermissions(InstanceOperation):
    """Exposes the get_group_permissions API on Users.
    """

    def get(self, _request , response, _appname, _modelname, pk):
        """Implements the API.
        """
        user = self.model.model.objects.get(pk=pk)
        response['group_permissions'] = list(user.get_group_permissions())
        response['all_permissions'] = list(user.get_all_permissions())




"""
    The standard Slumber server implementation.
"""
from urlparse import urljoin
from urlparse import urlparse

from django.conf import settings
from django.core.urlresolvers import reverse


class AbsoluteURIRequired(Exception):
    """Thrown when the local service is configured with a relative URL.
    """
    pass

class NoServiceSpecified(Exception):
    """Thrown when the service option for SLUMBER_DIRECTORY is used
    but no SLUMBER_SERVICE has been given.
    """
    pass

class NotAuthorised(Exception):
    """Thrown when an anonymous user tries to access an operation, model or
    application data.
    """
    pass

class Forbidden(Exception):
    """Thrown when a user tries to access an operation for which they don't
    have permission.
    """
    pass


def _get_slumber_service():
    """Implementation for get_slumber_service which allows a single
    patching point.
    """
    return getattr(settings, 'SLUMBER_SERVICE', None)
def get_slumber_service():
    """Returns the current Slumber service name.

    This allows us to control the setting value in tests more easily for
    early versions of Django.
    """
    return _get_slumber_service()


def _get_slumber_directory():
    """Implementation for get_slumber_directory which allows a single
    patching point.

    This allows us to control the setting value in tests more easily for
    early versions of Django.
    """
    return getattr(settings, 'SLUMBER_DIRECTORY',
        'http://localhost:8000/slumber/')
def get_slumber_directory():
    """Returns the directory setting.
    """
    return _get_slumber_directory()


def get_slumber_service_url():
    """Returns the full URL found in the settings for the main local service.
    """
    directory = get_slumber_directory()
    if hasattr(directory, 'items'):
        try:
            directory = directory[get_slumber_service()]
        except KeyError:
            raise NoServiceSpecified("If you have a Slumber directory "
                "specifying services you must also set SLUMBER_SERVICE")
    return directory


def get_slumber_local_url_prefix():
    """Returns the local URL prefix for Slumber access.
    """
    service_url = get_slumber_service_url()
    parsed = urlparse(service_url)
    scheme, netloc = parsed[0], parsed[1]
    if not scheme or not netloc:
        raise AbsoluteURIRequired("The URL for the local service must be "
        "specified as absolute: %s is currently %s" %
            (get_slumber_service(), service_url))
    return '%s://%s/' % (parsed[0], parsed[1])


def get_slumber_services(directory = None):
    """Returns the slumber services from the directory (if specified)
    """
    if not directory:
        directory = get_slumber_directory()
    if hasattr(directory, 'items'): # Feels like a dict
        services = {}
        for k, v in directory.items():
            if v in settings.INSTALLED_APPS:
                url = urljoin(
                    get_slumber_service_url(), v.replace('.', '/') + '/')
                services[k] = url
            else:
                services[k] = v
        return services
    else:
        service = get_slumber_service()
        if service:
            return {service: directory + '%s/' % service}
        return None


def get_slumber_root():
    """Returns the location of the Slumber on this server.
    """
    root = reverse('slumber.server.views.service_root')
    service = get_slumber_service()
    if service:
        return '%s%s/' % (root, service)
    else:
        return root



"""
    Wrappers for interacting with Django in the server.
"""
from django.conf import settings

from slumber._caches import APP_FROM_APPNAME
from slumber.server.application import DjangoApp


def applications():
    """Return the Django application wrappers for all installed apps.
    """
    if APP_FROM_APPNAME:
        return APP_FROM_APPNAME.values()
    else:
        apps = [get_application(app) for app in settings.INSTALLED_APPS]
        for app in apps:
            __import__(app.name, globals(), locals(), ['slumber_server'])
        return apps

def get_application(app_name):
    """Build a Django application wrapper around an application given
    by its name.
    """
    if not APP_FROM_APPNAME.has_key(app_name):
        APP_FROM_APPNAME[app_name] = DjangoApp(app_name)
    return APP_FROM_APPNAME[app_name]



"""
    Utility functions for managing the slumber URL scheme.
"""


class SlumberServiceURLError(Exception):
    """This exception is thrown if the Slumber URL has a service that does
    not match the service that it should have.
    """
    pass


def to_slumber_scheme(url, services):
    """Look at the URL and convert it to a service based URL if applicable.
    """
    ret = None
    if services:
        longest = None, ''
        for service, service_url in services.items():
            if url.startswith(service_url) and len(service_url) > len(longest):
                ret, longest =  \
                    'slumber://%s/%s' % (service, url[len(service_url):]), \
                        service_url
    return ret or url


def from_slumber_scheme(url, services):
    """Turn a Slumber URL into a full URI as specified by the service
    configuration.
    """
    if url.startswith('slumber://'):
        if not services:
            raise SlumberServiceURLError(
                "There are no services in the Slumber directory "
                "so the URL %s cannot be dereferenced" % url)
        for service in  services.keys():
            service_prefix = 'slumber://%s/' % service
            if url.startswith(service_prefix):
                service_url = services[service]
                return service_url + url[len(service_prefix):]
        raise SlumberServiceURLError(
            "Service in URL %s does not found in configured services %s"
                % (url, services.keys()))
    return url



"""
    Implements creation of an object.
"""
from slumber.operations import InstanceOperation
from slumber.server.http import require_permission


class DeleteInstance(InstanceOperation):
    """Allows for the removal of instances.
    """
    def post(self, request, response, appname, modelname, pk):
        """Perform the object deletion.
        """
        # We need all of these arguments as they are all used
        # pylint: disable=R0913
        @require_permission('%s.delete_%s' % (appname, modelname.lower()))
        def do_delete(_cls, _request):
            """This inner function is used to allow us to build a correct
            permission name at run time based on the application and model
            names.
            """
            instance = self.model.model.objects.get(pk=pk)
            instance.delete()
            response['deleted'] = True
        return do_delete(self, request)




from datetime import date
from simplejson import loads
from mock import patch

from django.test import TestCase

from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL
from slumber.server import get_slumber_services, get_slumber_local_url_prefix, \
    NoServiceSpecified, AbsoluteURIRequired
from slumber.server.http import view_handler
from slumber.server.meta import get_application

from slumber_examples.models import Pizza


class TestJSON(TestCase):
    def test_unicode_attributes(self):
        d = date.today()
        class Request(object):
            META = {}
            class user(object):
                @classmethod
                def is_authenticated(cls):
                    return True
                username = 'testuser'
        @view_handler
        def view(request, response):
            response['u'] = d
        http_response = view(Request())
        content = loads(http_response.content)
        self.assertEquals(content, dict(
            u = str(d),
            _meta = dict(status = 200, message = "OK", username = "testuser")))


class InternalAPIs(TestCase):
    def test_get_application(self):
        app = get_application('slumber_examples')
        self.assertEqual(app.name, 'slumber_examples')
        self.assertEqual(app.path, 'slumber_examples')

    def test_slumber_services_none(self):
        directory = get_slumber_services()
        self.assertIsNone(directory)

    def test_slumber_services(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': '/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            directory = get_slumber_services()
            self.assertIsNotNone(directory)

    def test_slumber_local_url(self):
        with patch('slumber.server.get_slumber_directory',
                lambda: 'http://example.com/somewhere/slumber/'):
            self.assertEqual(get_slumber_local_url_prefix(),
                'http://example.com/')

    def test_slumber_local_url_dev_server(self):
        with patch('slumber.server.get_slumber_directory',
                lambda: 'http://localhost:8008/somewhere/slumber/'):
            self.assertEqual(get_slumber_local_url_prefix(),
                'http://localhost:8008/')

    def test_slumber_local_url_https(self):
        with patch('slumber.server.get_slumber_directory',
                lambda: 'https://example.com/somewhere/slumber/'):
            self.assertEqual(get_slumber_local_url_prefix(),
                'https://example.com/')

    def test_slumber_local_url_with_services(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': 'http://localhost:8000:/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            with patch('slumber.server.get_slumber_service', lambda: 'pizzas'):
                self.assertEqual(get_slumber_local_url_prefix(),
                'http://localhost:8000:/')

    def test_slumber_local_url_with_services_but_no_service_specified(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': 'http://localhost:8000:/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            with self.assertRaises(NoServiceSpecified):
                self.assertEqual(get_slumber_local_url_prefix(),
                'http://localhost:8000:/')

    def test_slumber_local_url_with_services_but_relative_self_service_url(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': '/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            with patch('slumber.server.get_slumber_service', lambda: 'pizzas'):
                with self.assertRaises(AbsoluteURIRequired):
                    self.assertEqual(get_slumber_local_url_prefix(),
                    'http://localhost:8000:/')

    def test_application_repr(self):
        model = DJANGO_MODEL_TO_SLUMBER_MODEL[Pizza]
        self.assertEqual(str(model), "slumber_examples.Pizza")
        self.assertEqual(str(model.app), "slumber_examples")



"""
    Implements the JSON formatting for both the client and the server.
"""
from urlparse import urljoin


def from_json_data(base_url, json):
    """Convert a JSON representation of some data to the right types within
    the client.
    """
    if json['kind'] == 'object':
        if json['data'] is None:
            return None
        else:
            # It's a remote object
            from slumber.connector.api import get_instance, get_model
            model_url = urljoin(base_url, json['data']['type'])
            data_url = urljoin(base_url, json['data']['data'])
            display = json['data']['display']
            return get_instance(get_model(model_url), data_url, display)
    else:
        return json['data']



"""
    Slumber is a RESTful client/server implementation that makes it simple
    to make use of Django models in a RESTful manner.
"""
from slumber.connector import Client
from slumber.configuration import configure


# In order to allow mocking of the Slumber client instance we need an extra
# level of indirection to give us something we can mock. The below code
# provides that.

# Create a location for where to put the real client
_client = Client()

class _ClientProxy(object):
    """A forwarder for the real client.
    """
    def __getattr__(self, name):
        return getattr(_client, name)

# The client other code will import will always forward to the mockable
# _client instance.
# This is exactly the name we want to use here
# pylint: disable=C0103
client = _ClientProxy()



"""
    Implements the server side wrapper for a Django model.
"""
from django.db.models import ForeignKey, ManyToManyField
from django.db.models.fields import FieldDoesNotExist

from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL, \
    SLUMBER_MODEL_OPERATIONS
from slumber.operations.authenticate import AuthenticateUser
from slumber.operations.authorization import PermissionCheck, \
    ModulePermissions, GetPermissions
from slumber.operations.create import CreateInstance
from slumber.operations.delete import DeleteInstance
from slumber.operations.instancedata import InstanceData
from slumber.operations.instancelist import InstanceList
from slumber.operations.profile import GetProfile
from slumber.operations.search import DereferenceInstance
from slumber.operations.update import UpdateInstance
from slumber.server import get_slumber_root


class NotAnOperation(Exception):
    """Thrown when an operation is looked for by name, but doesn't exist.
    """
    pass


class DjangoModel(object):
    """Describes a Django model.
    """
    def __init__(self, app, model_instance):
        DJANGO_MODEL_TO_SLUMBER_MODEL[model_instance] = self
        self.app = app
        self.model = model_instance
        self.name = model_instance.__name__
        self.path = app.path + '/' + self.name + '/'

        self.properties = dict(r=[], w=[])
        self._fields, self._data_arrays = {}, []
        SLUMBER_MODEL_OPERATIONS[self] = [
            InstanceList(self, 'instances'),
            CreateInstance(self, 'create'),
            InstanceData(self, 'data'),
            DeleteInstance(self, 'delete'),
            DereferenceInstance(self, 'get'),
            UpdateInstance(self, 'update')]
        if self.path == 'django/contrib/auth/User/':
            ops = SLUMBER_MODEL_OPERATIONS[self]
            ops.append(AuthenticateUser(self, 'authenticate'))
            ops.append(PermissionCheck(self, 'has-permission'))
            ops.append(GetPermissions(self, 'get-permissions'))
            ops.append(GetProfile(self, 'get-profile'))
            ops.append(ModulePermissions(self, 'module-permissions'))

    def __repr__(self):
        return "%s.%s" % (self.app, self.name)

    def _get_fields_and_data_arrays(self):
        """Work out what the fields we have are.
        """
        if self._fields or self._data_arrays:
            return
        for field in self.model._meta.get_all_field_names():
            try:
                definition = self.model._meta.get_field(field)
                if type(definition) == ManyToManyField:
                    self._data_arrays.append(field)
                else:
                    self._fields[field] = definition
            except FieldDoesNotExist:
                self._data_arrays.append(field)

    @property
    def fields(self):
        """Return the non-array fields.
        """
        self._get_fields_and_data_arrays()
        fields = {}
        for field, definition in self._fields.items():
            field_type = type(definition)
            if field_type == ForeignKey:
                fields[field] = dict(
                    name=field,
                    kind='object',
                    type= get_slumber_root() +
                        DJANGO_MODEL_TO_SLUMBER_MODEL[definition.rel.to].path,
                    verbose_name=definition.verbose_name)
            else:
                type_name = field_type.__module__ + '.' + \
                    field_type.__name__
                fields[field] = dict(name=field,
                    kind='value', type=type_name,
                    verbose_name=definition.verbose_name)
        for prop in self.properties['r']:
            fields[prop] = dict(
                name=prop,
                kind='property',
                type='.'.join([self.app.name, self.name, prop]),
                readonly=True)
        return fields

    @property
    def data_arrays(self):
        """Return the data array fields.
        """
        self._get_fields_and_data_arrays()
        return self._data_arrays

    def operations(self):
        """Return all of  the operations available for this model.
        """
        return SLUMBER_MODEL_OPERATIONS[self]

    def operation_by_name(self, name):
        """Return a given operation by name, or throw an exception.
        """
        ops = [o for o in self.operations() if o.name == name]
        if len(ops) != 1:
            raise NotAnOperation("Operation %s not found (options %s)" %
                (name, ops))
        else:
            return ops[0]



# -*- coding: utf-8 -*-
"""
    The Django URLs for the server.
"""
from django.conf.urls.defaults import patterns


# The name urlpatterns is defined by Django and we can't change it
# pylint: disable=C0103
#urlpatterns = patterns('', *_urls.items())
urlpatterns = patterns('',
    ('.*', 'slumber.server.views.service_root'))



from mock import patch

from django.conf import settings
from django.contrib.auth.models import User

from slumber.connector import Client
from slumber.connector.ua import _calculate_signature


class ConfigureUser(object):
    def setUp(self):
        self.user = User(username='user', is_active=True, is_staff=True,
            is_superuser=False)
        self.user.set_password('pass')
        self.user.save()
        self.service = User(username='service', is_active=True, is_staff=True,
            is_superuser=True, password=settings.SECRET_KEY)
        self.service.save()
        self.__patchers = [
            patch('slumber.connector._get_slumber_authn_name', lambda: 'service'),
        ]
        [p.start() for p in self.__patchers]
        super(ConfigureUser, self).setUp()
    def tearDown(self):
        super(ConfigureUser, self).tearDown()
        [p.stop() for p in self.__patchers]

    def signed_get(self,  username, url='/'):
        headers = _calculate_signature('service', 'GET', url, '', username, True)
        return self.client.get(url, **headers)

    def signed_post(self, username, url, data):
        headers = _calculate_signature('service', 'POST', url, data, username, True)
        return self.client.post(url, data, **headers)


class ConfigureAuthnBackend(ConfigureUser):
    def setUp(self):
        self.assertFalse(hasattr(settings, 'SLUMBER_DIRECTORY'))
        self.assertFalse(hasattr(settings, 'SLUMBER_SERVICE'))
        self.__backends = settings.AUTHENTICATION_BACKENDS
        settings.AUTHENTICATION_BACKENDS = [
            'slumber.connector.authentication.Backend',
        ]
        super(ConfigureAuthnBackend, self).setUp()

    def tearDown(self):
        super(ConfigureAuthnBackend, self).tearDown()
        settings.AUTHENTICATION_BACKENDS = self.__backends


class PatchForAuthnService(ConfigureUser):
    def setUp(self):
        self.assertFalse(hasattr(settings, 'SLUMBER_DIRECTORY'))
        self.assertFalse(hasattr(settings, 'SLUMBER_SERVICE'))
        service = lambda: 'auth'
        directory = lambda: {
            'auth': 'http://localhost:8000/slumber/auth/',
        }
        self.__patchers = [
            patch('slumber.server._get_slumber_service', service),
            patch('slumber.server._get_slumber_directory', directory),
        ]
        [p.start() for p in self.__patchers]
        client_patch = patch('slumber._client', Client())
        client_patch.start()
        self.__patchers.append(client_patch)
        super(PatchForAuthnService, self).setUp()
        self.user = User.objects.get(username=self.user.username)
    def tearDown(self):
        super(PatchForAuthnService, self).tearDown()
        [p.stop() for p in self.__patchers]




"""
    Authentication backend that sends all of the permissions checks
    to a remote service.
"""
from django.contrib.auth.models import Permission, User
from django.contrib.contenttypes.models import ContentType

from fost_authn.authentication import FostBackend

from slumber import client
from slumber.connector.proxies import attach_to_local_user


class ImproperlyConfigured(Exception):
    """Raised if the authentication is not configured as it should be.
    """
    pass


def _assert_properly_configured():
    """Make sure that the authentication backend is properly configured.
    """
    if not hasattr(client, 'auth'):
        raise ImproperlyConfigured("If using the Slumber client's "
            "authentication backend then you must also use the full "
            "service based Slumber configuration and include a service "
            "called 'auth' which points to the service which will "
            "handle all authentication and authorization.")


class Backend(FostBackend):
    """An authentication backend which delegates user permissions to another
    Slumber service.

    Currently this backend does not support object permissions.
    """

    def authenticate(self, **kwargs):
        """Authenticate the user when the middleware passes it in.
        """
        _assert_properly_configured()
        user = super(Backend, self).authenticate(**kwargs)
        if not user:
            username = kwargs.get('username', None)
            password = kwargs.get('password', None)
            if username and password:
                user = client.auth.django.contrib.auth.User.authenticate(
                    username=username, password=password)
        return user


    # Django defines the following as methods
    # pylint: disable=R0201
    def get_user(self, user_id):
        """Return the user associated with the user_id specified.
        """
        _assert_properly_configured()
        if isinstance(user_id, int):
            local_user = User.objects.get(id=user_id)
            remote_user = \
                client.auth.django.contrib.auth.User.get(
                    **{'username':local_user.username})
        else:
            remote_user = \
                client.auth.django.contrib.auth.User.get(
                    **{'username':user_id})
        return attach_to_local_user(remote_user)

    def get_group_permissions(self, user_obj, _obj=None):
        """Returns all of the permissions the user has through their groups.
        """
        return user_obj.remote_user.get_group_permissions()

    def get_all_permissions(self, user_obj, _obj=None):
        """Return all of the permission names that this user has.
        """
        return user_obj.remote_user.get_all_permissions()

    def has_module_perms(self, user_obj, package_name):
        """Return True if the user has any permission within the
        module/application
        """
        return user_obj.remote_user.has_module_perms(package_name)

    def has_perm(self, user_obj, perm, _obj=None):
        """Return True if the user has the specified permission.
        """
        try:
            app, code = perm.split('.')
            # Pylint can't work out that objects exists on Permission
            # pylint: disable = E1101
            if Permission.objects.filter(codename=code,
                    content_type__app_label=app).count() == 0:
                content_type, _ = ContentType.objects.get_or_create(
                    app_label=app, model='unknown')
                Permission(codename=code, name=code,
                    content_type=content_type).save()
        except ValueError:
            pass
        return user_obj.remote_user.has_perm(perm)




"""
    Some caches used in the implementation of the Slumber client or server.
"""
import threading


# Stores the applications via their application names
APP_FROM_APPNAME = {}

# Stores the server model for a given Django model in the server
DJANGO_MODEL_TO_SLUMBER_MODEL = {}
# Stores the slumber models for given model URLs
MODEL_URL_TO_SLUMBER_MODEL = {}


# Stores instances in the client
CLIENT_INSTANCE_CACHE = type('cache', (dict,), {})()
# Leave the cache off by default
CLIENT_INSTANCE_CACHE.enabled = False


# Stores the operations used for a given model on the server
SLUMBER_MODEL_OPERATIONS = {}


# Make the request available
PER_THREAD = threading.local()



"""
    Allows a user to be authenticated.
"""
from django.contrib.auth import authenticate

from slumber.operations import ModelOperation
from slumber.server import get_slumber_root


class AuthenticateUser(ModelOperation):
    """Allows a user to be authenticated.
    """
    def post(self, request, response, _appname, _modelname):
        """Perform the authentication.
        """
        # This method can't be a function
        # pylint: disable=R0201
        user = authenticate(**dict([(str(k), str(v))
            for k, v in request.POST.items()]))
        response['authenticated'] = bool(user)
        if user:
            root = get_slumber_root()
            response['user'] = dict(
                pk = user.pk,
                display_name = unicode(user),
                url = root + self.model.path + 'data/%s/' % user.pk)
        else:
            response['user'] = None



"""
    Implements the JSON formatting for both the server.
"""
from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL
from slumber.server import get_slumber_root


DATA_MAPPING = {
        'django.db.models.fields.AutoField': lambda m, i, fm, v: v,
        'django.db.models.fields.BooleanField': lambda m, i, fm, v: v,
        'django.db.models.fields.NullBooleanField': lambda m, i, fm, v: v,
    }


def to_json_data(model, instance, fieldname, fieldmeta):
    """Convert a model field to JSON on the server.
    """
    value = getattr(instance, fieldname)
    if fieldmeta['kind'] == 'object':
        if value is None:
            return None
        else:
            rel_to = DJANGO_MODEL_TO_SLUMBER_MODEL[type(value)]
            root = get_slumber_root()
            return dict(type=root + rel_to.path,
                display=unicode(value),
                data = root + rel_to.path + 'data/%s/' % value.pk)
    elif DATA_MAPPING.has_key(fieldmeta['type']):
        func = DATA_MAPPING[fieldmeta['type']]
        return func(model, instance, fieldmeta, value)
    else:
        if value is None:
            return None
        else:
            return unicode(value)



"""
    Implements the server side for the instance operators.
"""
from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL
from slumber.operations import InstanceOperation
from slumber.server import get_slumber_root
from slumber.server.http import require_user
from slumber.server.json import to_json_data


def instance_data(into, model, instance):
    """Fill in the dict `into` with information about the instance of the
    specified model.
    """
    root = get_slumber_root()
    into['type'] = root + model.path
    into['identity'] = root + model.path + \
        '%s/%s/' % ('data', instance.pk)
    into['display'] = unicode(instance)
    into['operations'] = dict(
        [(op.name, root + op.path + '%s/' % instance.pk)
            for op in model.operations() if not op.model_operation])
    into['fields'] = {}
    for field, meta in model.fields.items():
        into['fields'][field] = dict(
            data=to_json_data(model, instance, field, meta),
            kind=meta['kind'], type=meta['type'])
    into['data_arrays'] = {}
    for field in model.data_arrays:
        into['data_arrays'][field] = \
            into['identity'] + '%s/' % field


class InstanceData(InstanceOperation):
    """Return the instance data.
    """
    @require_user
    def get(self, request, response, _appname, _modelname, pk, dataset = None):
        """Implement the fetching of attribute data for an instance.
        """
        instance = self.model.model.objects.get(pk=pk)
        if dataset:
            self._get_dataset(request, response, instance, dataset)
        else:
            self._get_instance_data(request, response, instance)

    def _get_instance_data(self, _request, response, instance):
        """Return the base field data for the instance.
        """
        return instance_data(response, self.model, instance)

    def _get_dataset(self, request, response, instance, dataset):
        """Return one page of the array data.
        """
        root = get_slumber_root()
        response['instance'] = root + self.model.path + '%s/%s/%s/' % (
            self.name, str(instance.pk), dataset)

        try:
            query = getattr(instance, dataset + '_set')
        except AttributeError:
            query = getattr(instance, dataset)
        query = query.order_by('-pk')
        if request.GET.has_key('start_after'):
            query = query.filter(pk__lt=request.GET['start_after'])

        response['page'] = []
        for obj in query[:10]:
            model = DJANGO_MODEL_TO_SLUMBER_MODEL[type(obj)]
            response['page'].append(dict(
                    type=root + model.path,
                    pk=obj.pk, display=unicode(obj),
                    data=root + model.path + 'data/%s/' % obj.pk))

        if query.count() > len(response['page']):
            response['next_page'] = root + self.model.path + \
                '%s/%s/%s/?start_after=%s' % (
                    self.name, instance.pk, dataset,
                    response['page'][-1]['pk'])



"""
    Some basic server views.
"""
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, \
    HttpResponsePermanentRedirect, HttpResponseNotFound

from slumber.server import get_slumber_service, get_slumber_root, \
    get_slumber_services
from slumber.server.http import view_handler
from slumber.server.meta import applications
from slumber.server.model import NotAnOperation


@view_handler
def service_root(request, response):
    """Request routing for Slumber.
    """
    # We have many return statements, but there's no point in artificially
    # breaking the function up to reduce them
    # pylint: disable = R0911
    if not request.path.endswith('/'):
        return HttpResponsePermanentRedirect(request.path + '/')
    path = request.path[len(reverse('slumber.server.views.service_root')):-1]
    service = get_slumber_service()
    if service:
        if not path:
            return get_service_directory(request, response)
        elif not path.startswith(service + '/') and path != service:
            return HttpResponseNotFound()
        path = path[len(service) + 1:]

    if not path:
        return get_applications(request, response)
    else:
        # Find the app with the longest matching path
        apps = [a for a in applications() if path.startswith(a.path)]
        application = None
        for app in apps:
            if not application or len(app.path) > len(application.path):
                application = app
        if not application:
            return HttpResponseNotFound()
        remaining_path = path[len(application.path)+1:]
        if not remaining_path:
            return get_models(request, response, application)

        models = remaining_path.split('/')
        # Find the model instance and possibly return its details
        model = application.models.get(models.pop(0), None)
        if len(models) == 0:
            return get_model(request, response, model)

        try:
            # Execute the operation (if it can be found)
            operation_name = models.pop(0)
            operation = model.operation_by_name(operation_name)
            return operation.operation(request, response,
                application.path, model.name, *models)
        except NotAnOperation:
            return HttpResponseNotFound()


def get_service_directory(_request, response):
    """Return the services.
    """
    response['services'] = get_slumber_services()


def get_applications(request, response):
    """Return the list of applications and the data connection URLs for them.
    """
    root = get_slumber_root()
    if request.GET.has_key('model'):
        appname, modelname = request.GET['model'].split('.')
        for app in applications():
            if app.name.endswith(appname) and app.models.has_key(modelname):
                return HttpResponseRedirect(root + app.models[modelname].path)
        return HttpResponseNotFound()
    response['apps'] = dict([(app.name, root + app.path + '/')
        for app in applications()])
    get_service_directory(request, response)


def get_models(_, response, app):
    """Return the models that comprise an application.
    """
    root = get_slumber_root()
    response['models'] = dict([(n, root + m.path)
        for n, m in app.models.items()])


def get_model(_, response, model):
    """Return meta data about the model.
    """
    if not model:
        return HttpResponseNotFound()
    root = get_slumber_root()
    response['name'] = model.name
    response['module'] = model.app.name
    response['fields'] = model.fields
    response['puttable'] = [[f] for f, p in model.fields.items()
            if p['kind'] != 'property' and
                model.model._meta.get_field(f).unique] + \
        list(model.model._meta.unique_together)
    response['data_arrays'] = model.data_arrays
    response['operations'] = dict(
        [(op.name, root + op.path)
            for op in model.operations() if op.model_operation])




"""
    Code for the Slumber client connector.
"""
import logging
from urllib import urlencode
from urlparse import urljoin

from django.conf import settings

from slumber._caches import CLIENT_INSTANCE_CACHE, \
    MODEL_URL_TO_SLUMBER_MODEL
from slumber.connector.api import get_model
from slumber.connector.dictobject import DictObject
from slumber.connector.json import from_json_data
from slumber.connector.ua import get
from slumber.server import get_slumber_service, get_slumber_directory, \
    get_slumber_services, get_slumber_local_url_prefix, get_slumber_root


def _get_slumber_authn_name():
    """Used in the implementation of get_auth_name so it can be easily
    patched.
    """
    return getattr(settings, 'SLUMBER_AUTHN_NAME', get_slumber_service())
def get_slumber_authn_name():
    """Return the user name that is to be used for authenticating Slumber
    requests to the back end. This user name is used together with the
    SECRET_KEY and defaults to the service name.
    """
    return _get_slumber_authn_name()


class ServiceConnector(object):
    """Connects to a service.
    """
    def __init__(self, directory):
        self._directory = directory

    def __getattr__(self, attr_name):
        """Fetch the application list from the Slumber directory on request.
        """
        logging.debug("Looking for attribute %s on %s for directory %s",
            attr_name, self, self._directory)
        if not self._directory:
            logging.debug("Raising AttributeError as _directory is falsey")
            raise AttributeError(attr_name)
        _, json = get(self._directory)
        logging.debug(
            "Looking for attribute %s on %s resulted in these applications %s",
            attr_name, self, json)
        # Pylint gets confused by the JSON object
        # pylint: disable=E1103
        json_apps = json.get('apps', {})
        apps = {}
        for app in json_apps.keys():
            root = apps
            for k in app.split('.'):
                if not root.has_key(k):
                    root[k] = {}
                root = root[k]
        def recurse_apps(loc, this_level, name):
            """Recursively build the application connectors.
            """
            current_appname = '.'.join(name)
            if json_apps.has_key(current_appname):
                loc._directory = urljoin(self._directory,
                    json_apps[current_appname])
            for k, v in this_level.items():
                app_cnx = ServiceConnector(None)
                setattr(loc, k, app_cnx)
                recurse_apps(app_cnx, v, name + [k])
        recurse_apps(self, apps, [])
        models = json.get('models', {})
        for model_name, url in models.items():
            model_url = urljoin(self._directory, url)
            model = get_model(model_url)
            setattr(self, model_name, model)
        if attr_name in models.keys():
            return getattr(self, attr_name)
        elif attr_name in apps.keys():
            return getattr(self, attr_name)
        else:
            raise AttributeError(attr_name, json)


class Client(ServiceConnector):
    """The first level of the Slumber client connector.
    """
    def __init__(self, directory=None):
        client_apps = getattr(settings, 'SLUMBER_CLIENT_APPS', [])
        for app in client_apps:
            __import__(app, globals(), locals(), ['slumber_client'])
        services = get_slumber_services(directory)
        if not services:
            if not directory:
                directory = get_slumber_directory()
            super(Client, self).__init__(directory)
        else:
            for k, v in services.items():
                setattr(self, k, ServiceConnector(v))
            super(Client, self).__init__(None)

    @classmethod
    def _flush_client_instance_cache(cls):
        """Flush the (global) instance cache.
        """
        CLIENT_INSTANCE_CACHE.clear()



"""
    Used to allow simple initialisation of Python objects contains more or
    less any members.
"""

class DictObject(object):
    """Allows generic Python objects to be created from a nested dict
    structure describing the attrbutes.
    """
    def __init__(self, **kwargs):
        """Load the specified key values.
        """
        super(DictObject, self).__init__()
        proc = lambda value: DictObject(**value) \
            if hasattr(value, 'items') else value
        for key, value in kwargs.items():
            if hasattr(value, 'count') and not hasattr(value, 'capitalize'):
                setattr(self, key, [proc(i) for i in value])
            else:
                setattr(self, key, proc(value))




from httplib2 import ServerNotFoundError
from mock import Mock, patch
import socket
from unittest2 import TestCase

from django.core.cache import cache

from slumber.connector.ua import get, post


class _response_fake(object):
    status_code = 200
    content = '123'

class _response_httplib2(object):
    status = 200
    content = '123'


class TestPost(TestCase):

    def test_fake(self):
        def _post(self, url, data, **kw):
            return _response_fake()
        with patch('slumber.connector.ua.FakeClient.post', _post):
            response, json = post('/local/', {})
        self.assertEqual(json, 123)

    def test_real(self):
        def _request(_self, url, method, body, headers={}):
            self.assertEqual(body, '{"data": 23}')
            return _response_httplib2(), "123"
        with patch('slumber.connector.ua.Http.request', _request):
            response, json = post('http://example.com/', {
                'data': 23})
        self.assertEqual(json, 123)

    def test_404_allowed_for_fake(self):
        response, json = post('/slumber/does-not-exist/', {}, [404])
        self.assertEqual(response.status_code, 404)


class TestGet(TestCase):
    def setUp(self):
        self.cache_url = 'http://example.com'
    def tearDown(self):
        cache.delete('slumber.connector.ua.get.' + self.cache_url)

    def test_real(self):
        class response:
            status = 200
            content = '''{"apps":{}}'''
        def _request(_self, url, headers={}):
            r = response()
            return r, r.content
        with patch('slumber.connector.ua.Http.request', _request):
            response, json = get(self.cache_url)
        self.assertEqual(json, {"apps":{}})

    def test_real_does_not_use_cached_httplib2_client(self):
        """
        When we use this slumber in Wimbledon project, we found that the request
        is occationally error on IE9 (got 500 response in return bcoz of the empty
        request). We are not sure if the root cause was in IE9 or in the httplib2,
        but stop caching httplib2's client fixed it (reset client everytime).

        The downside is we cannot use caching features of the httplib2's client,
        but until we understand this problem in more details, we'll stick with
        resetting client every time.

        juacompe, 2nd November 2011

        ref: http://proteus.eidos.proteus-tech.com/project/BBG/story/BBG-251/card/
        """
        self.callers = []
        class response:
            status = 200
            content = '''{"apps":{}}'''
        def _request(_self, url, headers={}):
            self.callers.append(_self)
            r = response()
            return r, r.content
        with patch('slumber.connector.ua.Http.request',_request):
            get(self.cache_url)
            get(self.cache_url)
            self.assertNotEqual(self.callers[0], self.callers[1])

    def test_get_retries(self):
        class _response:
            content = '''{"apps":{}}'''
            def __init__(self, counter):
                self.status = 200 if counter == 1 else 404
        counter = []
        def _request(_self, url, headers={}):
            r = _response(len(counter))
            counter.append(True)
            return r, r.content
        with patch('slumber.connector.ua.Http.request', _request):
            get(self.cache_url)
        self.assertEqual(len(counter), 2)

    def test_retry_get_still_can_fail(self):
        class _response:
            content = ''
            status = 500
        def _request(_self, url, headers={}):
            r = _response()
            return r, r.content
        with patch('slumber.connector.ua.Http.request',_request):
            with self.assertRaises(AssertionError):
                get(self.cache_url)

    def test_cache_ttl(self):
        def _request(_self, url, headers={}):
            r = _response_httplib2()
            return r, r.content
        with patch('slumber.connector.ua.Http.request', _request):
            get(self.cache_url, 2)
        with patch('slumber.connector.ua.Http.request', self.fail):
            get(self.cache_url, 2)

    def test_cache(self):
        try:
            r1, j1 = get('http://urquell-fn.appspot.com/lib/echo/?__=', 2)
            r2, j2 = get('http://urquell-fn.appspot.com/lib/echo/?__=', 2)
            r3, j3 = get('http://urquell-fn.appspot.com/lib/echo/?dummy=&__=', 2)
            self.assertFalse(hasattr(r1, 'from_cache'))
            self.assertTrue(hasattr(r2, 'from_cache'))
            self.assertTrue(r2.from_cache)
            self.assertFalse(hasattr(r3, 'from_cache'))
        except ServerNotFoundError:
            # If we get a server error then we presume that there is no good
            # Internet connection and don't fail the test
            pass
        except socket.error:
            pass
        except Exception, e:
            print type(e)
            raise e




"""
    Implements the Django application wrapper for the Slumber server.
"""
from slumber.server.model import DjangoModel


class DjangoApp(object):
    """Describes a Django application.
    """
    def __init__(self, appname):
        self.name = appname
        self.path = appname.replace('.', '/')
        self.module = __import__(appname, globals(), locals(), ['models'])
        self.models = {}
        if hasattr(self.module, 'models'):
            for name in self.module.models.__dict__.keys():
                potential = getattr(self.module.models, name)
                if hasattr(potential, '_meta') and (
                        appname == potential._meta.app_label or
                        appname.endswith('.' + potential._meta.app_label)):
                    model = DjangoModel(self, potential)
                    self.models[name] = model

    def __repr__(self):
        return self.name



from django.db import models

from slumber.fields import RemoteForeignKey


class Shop(models.Model):
    name = models.fields.CharField(max_length=200)
    slug = models.fields.CharField(max_length=20, unique=True, blank=False)
    active = models.fields.NullBooleanField()

    @property
    def web_address(self):
        return 'http://www.example.com/%s/' % self.slug

    def __unicode__(self):
        return self.name


class Pizza(models.Model):
    name = models.fields.CharField(max_length=200, unique=True, blank=False)
    for_sale = models.fields.BooleanField()
    max_extra_toppings = models.fields.IntegerField(null=True, blank=False)
    exclusive_to = models.ForeignKey(Shop, null=True,
        help_text="If specified then this pizza is exclusive to the specified shop")

    def __unicode__(self):
        return self.name


class PizzaPrice(models.Model):
    pizza = models.ForeignKey(Pizza, null=False, related_name='prices')
    date = models.fields.DateField()


PIZZA_SIZES = (
    ('s', 'Small'),
    ('m', 'Medium'),
    ('l', 'Large'),
)

class PizzaSizePrice(models.Model):
    price = models.ForeignKey(PizzaPrice, null=False, related_name='amounts')
    amount = models.fields.DecimalField(decimal_places=2, max_digits=8)
    size = models.CharField(max_length=1, choices=PIZZA_SIZES)

    class Meta:
        unique_together=[('price', 'size')]


class Profile(models.Model):
    user = models.ForeignKey('auth.User',
        null=False, blank=False)


class Order(models.Model):
    shop = RemoteForeignKey(null=False,
        model_url='slumber://pizzas/slumber_examples/Shop/')



"""
    Proxies are used as base types for instances so that new APIs can be added.
"""
from urlparse import urljoin

from django.contrib.auth.models import User

from slumber.connector.api import get_instance_from_data
from slumber.connector.configuration import INSTANCE_PROXIES, MODEL_PROXIES
from slumber.connector.ua import get, post


def attach_to_local_user(remote_user):
    """Return the local user with the remote user object attached to it.
    """
    user, _ = User.objects.get_or_create(username=remote_user.username)
    for attr in ['is_active', 'is_staff', 'date_joined', 'is_superuser',
            'first_name', 'last_name', 'email']:
        v = getattr(remote_user, attr)
        setattr(user, attr, v)
    user.save()
    user.remote_user = remote_user
    # This lambda is necessary, but no idea why
    # pylint: disable = W0108
    user.get_profile = lambda: remote_user.get_profile()
    return user


class UserInstanceProxy(object):
    """Proxy that allows forwarding of the User API.
    """
    def __init__(self, *a, **kw):
        super(UserInstanceProxy, self).__init__(*a, **kw)
        self._CACHE_TTL = 120

    def has_perm(self, permission):
        """Forward the permission check.
        """
        # We're accessing attributes that are provided by the  other types
        # pylint: disable = E1101
        _, json = get(
            urljoin(self._operations['has-permission'], permission),
            self._CACHE_TTL)
        return json['is-allowed']

    def has_module_perms(self, module):
        """Forward the permission check.
        """
        # We're accessing attributes that are provided by the  other types
        # pylint: disable = E1101
        _, json = get(
            urljoin(self._operations['module-permissions'], module),
            self._CACHE_TTL)
        return json['has_module_perms']

    def get_group_permissions(self):
        """Forward the group permissions.
        """
        # We're accessing attributes that are provided by the  other types
        # pylint: disable = E1101
        _, json = get(self._operations['get-permissions'], self._CACHE_TTL)
        return set(json['group_permissions'])

    def get_all_permissions(self):
        """Forward access to all of the permissions.
        """
        # We're accessing attributes that are provided by the  other types
        # pylint: disable = E1101
        _, json = get(self._operations['get-permissions'], self._CACHE_TTL)
        return set(json['all_permissions'])

    def get_profile(self):
        """Forward access to the profile.
        """
        # We're accessing attributes that are provided by the  other types
        # pylint: disable = E1101
        base_url = self._operations['get-profile']
        _, json = get(base_url, self._CACHE_TTL)
        return get_instance_from_data(base_url, json)

INSTANCE_PROXIES['django/contrib/auth/User/'] = UserInstanceProxy


class UserModelProxy(object):
    """Contains the model methods that need to be exposed within the client.
    """
    def __init__(self, *a, **kw):
        super(UserModelProxy, self).__init__(*a, **kw)
        self._CACHE_TTL = 120

    def authenticate(self, **kwargs):
        """Allow a forwarded request for authentication.
        """
        # We're accessing attributes that are providec by the  other types
        # pylint: disable = E1101
        _, json = post(self._operations['authenticate'], kwargs)
        if json['authenticated']:
            # Pylint can't see the __call__ implemented in another base class
            # pylint: disable = E1102
            remote_user = self(urljoin(self._url, json['user']['url']),
                json['user']['display_name'])
            return attach_to_local_user(remote_user)

MODEL_PROXIES['django/contrib/auth/User/'] = UserModelProxy




"""
    Implements the user agent used to communicate with the Slumber
    servers.
"""
from django.conf import settings
from django.core.cache import cache
from django.test.client import Client as FakeClient, encode_multipart, \
    BOUNDARY
from django.utils.http import urlencode

from datetime import datetime
from fost_authn.signature import fost_hmac_request_signature
from httplib2 import Http
import logging
from simplejson import dumps, JSONDecodeError, loads
from urllib import urlencode
from urlparse import parse_qs, urlparse

from slumber._caches import PER_THREAD
from slumber.server import get_slumber_local_url_prefix


_fake = FakeClient()
def _real():
    """Don't check certificates when we use httplib2.
    """
    return Http(disable_ssl_certificate_validation=True)


def _parse_qs(url):
    """Split the query string off (this is needed to support Django 1.0's
    fake HTTP client.
    """
    if url.find('?') >= 0:
        path, query_string = url.split('?')
        return path, parse_qs(query_string)
    else:
        return url, {}


def _use_fake(url):
    """Return the local URL fragment if the request should use the Fake
    HTTP client as it is local, otherwise return None
    """
    slumber_local = get_slumber_local_url_prefix()
    if url.startswith(slumber_local):
        logging.debug("Using local fake HTTP due to %s starting with %s",
            url, slumber_local)
        return url[len(slumber_local) - 1:]
    elif url.startswith('/'):
        logging.debug("Using local fake HTTP due to %s starting with /",
            url)
        return url
    logging.debug("Using real HTTP for %s", url)


def _calculate_signature(authn_name, method, url, body,
        username, for_fake_client):
    """Do the signed request calculation.
    """
    # We need all arguments and all locals
    # pylint: disable=R0913
    # pylint: disable=R0914
    to_sign = {}
    if username:
        to_sign['X-FOST-User'] = username
    if not isinstance(body, basestring):
        if method in ['POST', 'PUT']:
            logging.info("Encoding POST/PUT data %s", body or {})
            data = encode_multipart(BOUNDARY, body or {})
        else:
            logging.info("Encoding query string %s", body or {})
            data = urlencode(body or {}, doseq=True)
    else:
        data = body or ''
    now = datetime.utcnow().isoformat() + 'Z'
    _, signature = fost_hmac_request_signature(
        settings.SECRET_KEY, method, url, now, to_sign, data)
    headers = {}
    headers['Authorization'] = 'FOST %s:%s' % (authn_name, signature)
    headers['X-FOST-Timestamp'] = now
    headers['X-FOST-Headers'] = ' '.join(['X-FOST-Headers'] + to_sign.keys())
    for key, value in to_sign.items():
        headers[key] = value
    logging.debug("_calculate_signature %s adding headers: %s", method, headers)
    if for_fake_client:
        return dict([('HTTP_' + k.upper().replace('-', '_'), v)
            for k, v in headers.items()])
    else:
        return headers


def _sign_request(method, url, body, for_fake_client):
    """Calculate the request headers that need to be added so that the
    request is properly signed and the Slumber server will consider
    the current user to be authenticated.
    """
    # import here avoids circular import
    from slumber.connector import get_slumber_authn_name
    authn_name = get_slumber_authn_name()
    if authn_name:
        username = getattr(PER_THREAD, 'username', None)
        return _calculate_signature(
            authn_name, method, url, body, username, for_fake_client)
    else:
        return {}


def get(url, ttl = 0, codes=None):
    """Perform a GET request against a Slumber server.
    """
    # Pylint gets confused by the fake HTTP client
    # pylint: disable=E1103
    url_fragment = _use_fake(url)
    if url_fragment:
        file_spec, query = _parse_qs(url_fragment)
        headers = _sign_request('GET', file_spec, query, True)
        response = _fake.get(file_spec, query,
            HTTP_HOST='localhost:8000', **headers)
        if response.status_code in [301, 302]:
            return get(response['location'])
        assert response.status_code in (codes or [200]), \
            (url_fragment, response, response.content)
        content = response.content
    else:
        cache_key = 'slumber.connector.ua.get.%s' % url
        cached = cache.get(cache_key)
        if not cached:
            logging.debug("Cache miss for url %s with cache key %s",
                url, cache_key)
            _, _, path, _, query, _ = urlparse(url)
            to_sign = path + ('' if not query else '?' + query)
            for _ in range(0, 3):
                headers = _sign_request('GET', to_sign, '', False)
                response, content = _real().request(
                    url, headers=headers)
                if response.status in (codes or [200]):
                    break
            assert response.status in (codes or [200]), \
                (url, response, content)
            if ttl:
                cache.set(cache_key, (response, content), ttl)
        else:
            logging.debug("Fetched %s from cache key %s", url, cache_key)
            response, content = cached
            response.from_cache = True
    return response, loads(content)


def post(url, data, codes=None):
    """Perform a POST request against a Slumber server.
    """
    # Pylint gets confused by the urlparse return type
    # pylint: disable=E1101
    # Pylint gets confused by the fake HTTP client
    # pylint: disable=E1103
    body = dumps(data) if data else ''
    url_fragment = _use_fake(url)
    if url_fragment:
        response = _fake.post(url_fragment, body,
            content_type='application/json',
            HTTP_HOST='localhost:8000',
            **_sign_request('POST', url_fragment, body, True))
        assert response.status_code in (codes or [200]), \
            (url_fragment, response, response.content)
        content = response.content
    else:
        headers = _sign_request('POST', urlparse(url).path, body, False)
        headers['Content-Type'] = 'application/json'
        response, content = _real().request(url, "POST", body=body,
            headers = headers)
        assert response.status in (codes or [200]), \
            (url, response, content)
    try:
        return response, loads(content)
    except JSONDecodeError:
        return response, {}




"""
    Implements updating of instances.
"""
from slumber.operations import InstanceOperation
from slumber.server import get_slumber_root
from slumber.server.http import require_permission


class UpdateInstance(InstanceOperation):
    """Update the attributes of a given instance.
    """
    def post(self, request, response, _app, _model, pk):
        """Perform the update.
        """
        @require_permission('%s.change_%s' % (
            self.model.app.name, self.model.name.lower()))
        def do_update(_, request):
            """A function we can decorate which will allow us to use the
            decorator with a dynamic permission name.
            """
            instance = self.model.model.objects.get(pk=pk)
            for k, v in request.POST.items():
                setattr(instance, k, v)
            instance.save()
            response['self'] = dict(url=get_slumber_root() +
                self.model.path + 'data/%s/' % instance.pk)
        return do_update(self, request)



from slumber import configure

from models import Pizza, Shop
from operations import OrderPizza


configure(Pizza,
    operations_extra = [(OrderPizza, 'order')])

configure(Shop,
    properties_ro = ['web_address'])



from django.test import TestCase

from slumber import client, configure
from slumber.server.application import DjangoApp

from slumber_examples.models import Shop
from slumber_examples.tests.configurations import ConfigureUser


from views import _perform


class TestSlumberConfiguration(ConfigureUser, TestCase):
    def test_shop_class_appears(self):
        self.assertTrue(hasattr(client.slumber_examples, 'Shop'),
            client.slumber_examples.__dict__.keys())

    def test_shop_model_has_web_address_property(self):
        response, json = _perform(self.client, 'get',
            '/slumber/slumber_examples/Shop/', {})
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['fields'].has_key('web_address'), json['fields'].keys())
        self.assertEquals(json['fields']['web_address']['kind'], 'property')
        self.assertEquals(json['fields']['web_address']['type'],
            'slumber_examples.Shop.web_address')
        self.assertTrue(json['fields']['web_address']['readonly'])

    def test_shop_instance_has_web_address(self):
        shop = Shop(name='Test', slug='test')
        shop.save()
        self.assertEquals(shop.web_address,
            'http://www.example.com/test/')
        rshop = client.slumber_examples.Shop.get(pk=shop.pk)
        self.assertEquals(rshop.slug, 'test')
        self.assertEquals(rshop.web_address,
            'http://www.example.com/test/')

    def test_to_json_configuration(self):
        shop = Shop(name='Test', slug='test')
        shop.save()
        configure(Shop,
            to_json = {
                'slumber_examples.Shop.web_address':
                    lambda m, i, fm, v: v.replace('http:', 'https:')
                })
        rshop = client.slumber_examples.Shop.get(pk=shop.pk)
        self.assertEquals(rshop.slug, 'test')
        self.assertEquals(rshop.web_address,
            'https://www.example.com/test/')
        from slumber.server.json import DATA_MAPPING
        del DATA_MAPPING['slumber_examples.Shop.web_address']



"""
    A test version of the client.
"""
import mock

from slumber.connector.dictobject import DictObject


def _do_get(model, **query):
    """Implements a mocked version of the get operator.
    """
    for i in model.instances:
        found = True
        for k, v in query.items():
            found = found and getattr(i, k) == v
        if found:
            return i
    assert False, "The instance was not found"


class _MockInstance(DictObject):
    """A mock instance that will add in a _url parameter.
    """
    def __init__(self, model_path, **kwargs):
        super(_MockInstance, self).__init__(**kwargs)
        if hasattr(self, 'pk'):
            self._url = ('slumber://' + model_path.replace('__', '/')
                + '/data/%s/' % getattr(self, 'pk'))

    def __repr__(self):
        return getattr(self, '_url', 'Unkown mock instance')


class _MockClient(DictObject):
    """Mock slumber client class.
    """
    def __init__(self, **instances):
        super(_MockClient, self).__init__()
        for model, instances in instances.items():
            root = self
            for k in model.split('__')[:-1]:
                if not hasattr(root, k):
                    setattr(root, k, DictObject())
                root = getattr(root, k)
            model_name = model.split('__')[-1]
            model_type = type(model_name, (_MockInstance,), {})
            setattr(model_type, 'instances',
                [model_type(model, **i) for i in instances])
            setattr(model_type, 'get', classmethod(_do_get))
            setattr(root, model_name, model_type)

    def _flush_client_instance_cache(self):
        """Empty stub so that the middleware works in tests.
        """


def mock_client(**instances):
    """Replaces the client with a mocked client that provides access to the
    provided applications, models and instances.
    """
    models = _MockClient(**instances)
    def decorator(test_method):
        """The actual decorator that is going to be used on the test method.
        """
        @mock.patch('slumber._client', models)
        def test_wrapped(test, *a, **kw):
            """The wrapper for the test method.
            """
            test_method(test, *a, **kw)
        test_wrapped.__doc__ = test_method.__doc__
        return test_wrapped
    return decorator



from mock import patch

from django.test import TestCase

from slumber.connector import Client, ServiceConnector

from slumber_examples.tests.configurations import ConfigureUser


class TestServices(ConfigureUser, TestCase):
    def setUp(self):
        self.service_name = lambda: 'pizzas'
        self.services = lambda _ = None: {
            self.service_name(): 'http://localhost:8000:/slumber/pizzas/',
            'takeaway': 'http://localhost:8002:/slumber/'}
        self.__patchers = [
            patch('slumber.server._get_slumber_directory', self.services),
            patch('slumber.server._get_slumber_service', self.service_name),
        ]
        [p.start() for p in self.__patchers]
        self.cnx = Client() # We need a client set up after we patch the directory
        super(TestServices, self).setUp()

    def tearDown(self):
        super(TestServices, self).tearDown()
        [p.stop() for p in self.__patchers]


    def test_services_appear(self):
        self.assertTrue(hasattr(self.cnx, 'pizzas'))
        self.assertTrue(hasattr(self.cnx, 'takeaway'))
        self.assertFalse(hasattr(self.cnx, 'not-a-service'))

    def test_service_has_correct_directory_url(self):
        self.assertEqual(self.cnx.pizzas._directory,
            'http://localhost:8000:/slumber/pizzas/')

    def test_service_applications_appear(self):
        self.assertTrue(
            isinstance(self.cnx.pizzas, ServiceConnector),
            type(self.cnx.pizzas))
        self.cnx.pizzas.slumber_examples
        self.assertTrue(
            hasattr(self.cnx.pizzas, 'slumber_examples'),
            self.cnx.pizzas.__dict__)




import os
from setuptools import setup

def read(fname1, fname2):
    if os.path.exists(fname1):
        fname = fname1
    else:
        fname = fname2
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "django_slumber",
    version = "0.6.1",
    author = "Kirit Saelensminde",
    author_email = "kirit@felspar.com",
    url='https://github.com/KayEss/django-slumber',
    description = ("RESTful data connector for Django"),
    long_description = read('README','README.markdown'),
    license = "Boost Software License - Version 1.0 - August 17th, 2003",
    keywords = "django rest data server client",
    packages = [
        'slumber', 'slumber.connector', 'slumber.operations', 'slumber.server',
        'slumber_examples', 'slumber_examples.no_models', 'slumber_examples.tests',
        'slumber_examples.nested1', 'slumber_examples.nested1.nested2',
        'slumber_ex_shop'],
    install_requires = ['simplejson', 'httplib2', 'django-fost-authn >= 0.3.4'],
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved",
    ],
)



from mock import patch
from simplejson import loads

from django.test import TestCase

from slumber import Client

from slumber_examples.models import Pizza
from slumber_examples.tests.configurations import ConfigureUser


class OrderTests(ConfigureUser, TestCase):
    def setUp(self):
        super(OrderTests, self).setUp()
        self.pizza = Pizza(name='S1', for_sale=True)
        self.pizza.save()
        self.cnx = Client()

    def test_order_get(self):
        with patch('slumber.connector._get_slumber_authn_name', lambda: 'service'):
            pizza = self.cnx.slumber_examples.Pizza.get(id=self.pizza.id)
            self.assertEquals(pizza._operations['order'],
                'http://localhost:8000/slumber/slumber_examples/Pizza/order/1/')
            # Do an unsigned GET so there will be no user logged in
            response = self.client.get('/slumber/slumber_examples/Pizza/order/1/')
            self.assertEquals(response.status_code, 200, response.content)
            json = loads(response.content)
            self.assertEquals(json, dict(
                    _meta = dict(status=200, message="OK"),
                    form = dict(quantity='integer'),
                ))

    def test_order_post(self):
        pizza = self.cnx.slumber_examples.Pizza.get(id=self.pizza.id)
        response = self.client.post('/slumber/slumber_examples/Pizza/order/1/',
            dict(quantity=3))
        self.assertEquals(response.status_code, 501, response.content)

    def test_order_create_with_pk(self):
        pizza2 = self.cnx.slumber_examples.Pizza.create(id=2, name='S2', for_sale=True)
        pizza3 = Pizza.objects.create(name='S3', for_sale=True)
        self.assertEqual(Pizza.objects.count(), 3)



from django.http import HttpResponse


def _ok_text(request):
    # Patch this if we want to patch the view function
    return HttpResponse("ok", 'text/plain')
def ok_text(request):
    return _ok_text(request)



from django.test import TestCase

from slumber import client
from slumber.connector import Client
from slumber.connector.proxies import UserInstanceProxy
from slumber_examples.models import Pizza
from slumber_examples.tests.configurations import PatchForAuthnService


class UserProxyTests(PatchForAuthnService, TestCase):
    def setUp(self):
        super(UserProxyTests, self).setUp()
        self.proxy_user = client.auth.django.contrib.auth.User.get(username='user')

    def test_user_proxy_is_used(self):
        self.assertIn(UserInstanceProxy, type(self.proxy_user).__mro__)

    def test_has_perm(self):
        self.assertFalse(self.proxy_user.has_perm('slumber_examples.not-a-permission'))

    def test_user_has_module_permission(self):
        self.assertFalse(self.proxy_user.has_module_perms('slumber_examples'))

    def test_user_get_group_permissions(self):
        perms = self.proxy_user.get_group_permissions()
        self.assertEqual(self.user.get_group_permissions(), perms)

    def test_user_get_all_permissions(self):
        perms = self.proxy_user.get_all_permissions()
        self.assertEqual(self.user.get_all_permissions(), perms)


class ProxyConfigurationTests(TestCase):
    def test_shop_has_model_proxy(self):
        self.assertTrue(
            client.slumber_examples.Shop.has_shop_proxy())

    def test_pizza_has_instance_proxy(self):
        lpizza = Pizza(name='Test pizza')
        lpizza.save()
        rpizza = client.slumber_examples.Pizza.get(
            pk=lpizza.pk)
        self.assertTrue(rpizza.has_pizza_proxy())



from django.test import TestCase as DBTestCase
from mock import patch
from unittest2 import TestCase

from slumber import client
from slumber.connector import Client
from slumber.scheme import to_slumber_scheme, from_slumber_scheme, \
    SlumberServiceURLError

from slumber_examples.models import Shop, Order
from slumber_examples.tests.configurations import ConfigureUser
from slumber_examples.tests.views import ServiceTestsWithDirectory


class URLHandlingToDatabase(TestCase):
    def test_no_services_pass_url_unchanged(self):
        translated = to_slumber_scheme(
            'http://example.com/slumber/', None)
        self.assertEquals(translated,
            'http://example.com/slumber/')

    def test_not_a_service(self):
        translated = to_slumber_scheme(
            'http://example.org/slumber/',
            dict(testservice='http://example.com/slumber/testservice/'))
        self.assertEquals(translated,
            'http://example.org/slumber/')

    def test_is_a_service(self):
        translated = to_slumber_scheme(
            'http://example.com/slumber/testservice/Model/',
            dict(testservice='http://example.com/slumber/testservice/'))
        self.assertEquals(translated,
            'slumber://testservice/Model/')

    def test_ambiguous_prefix(self):
        translated = to_slumber_scheme(
            'http://www.example.com/slumber/testservice/Model/',
            dict(a='http://www.example.com/slumber/testservice/',
                testmodel='http://www.example.com/slumber/testservice/Model/'))
        self.assertEquals(translated, 'slumber://testmodel/')


class URLHandlingFromDatabase(TestCase):
    def test_no_services_with_normal_url(self):
        translated = from_slumber_scheme(
            'http://example.com/slumber/', None)
        self.assertEquals(translated,
            'http://example.com/slumber/')

    def test_no_services_with_slumber_url(self):
        with self.assertRaises(SlumberServiceURLError):
            translated = from_slumber_scheme(
                'slumber://service/Model/', None)

    def test_not_a_slumber_url(self):
        translated = from_slumber_scheme(
            'http://example.org/slumber/',
            dict(testservice='http://example.com/slumber/testservice/'))
        self.assertEquals(translated,
            'http://example.org/slumber/')

    def test_slumber_service_url(self):
        translated = from_slumber_scheme(
            'slumber://testservice/Model/',
            dict(testservice='http://example.com/slumber/testservice/'))
        self.assertEquals(translated,
            'http://example.com/slumber/testservice/Model/')

    def test_slumber_service_url_with_a_different_service(self):
        translated = from_slumber_scheme(
            'slumber://another_service/Model/',
            dict(testservice='http://example.com/slumber/testservice/',
                another_service='http://example.com/slumber/another_service/'))
        self.assertEquals(translated,
            'http://example.com/slumber/another_service/Model/')

    def test_slumber_service_url_with_invalid_service(self):
        with self.assertRaises(SlumberServiceURLError):
            from_slumber_scheme(
                'slumber://not-a-service/Model/',
                dict(testservice='http://example.com/slumber/testservice/'))


class TestQueries(ConfigureUser, ServiceTestsWithDirectory, DBTestCase):
    REAL_URL = 'http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/'
    SLUMBER_URL = 'slumber://pizzas/slumber_examples/Shop/data/1/'

    def setUp(self):
        super(TestQueries, self).setUp()
        self.shop = Shop(name='Test', slug='test', active=True)
        self.shop.save()

    def test_have_shop(self):
        with patch('slumber._client', Client()):
            shop = client.pizzas.slumber_examples.Shop.get(pk=1)
            self.assertEqual(shop._url, self.REAL_URL)

    def test_find_order_by_shop_remote_instance(self):
        with patch('slumber._client', Client()):
            shop = client.pizzas.slumber_examples.Shop.get(pk=1)
            order = Order(shop=shop)
            order.save()
            fetched = Order.objects.get(shop=shop)
            self.assertEqual(order, fetched)

    def test_find_order_by_shop_url(self):
        with patch('slumber._client', Client()):
            shop = client.pizzas.slumber_examples.Shop.get(pk=1)
            order = Order(shop=shop)
            order.save()
            fetched = Order.objects.get(shop=self.SLUMBER_URL)
            self.assertEqual(order, fetched)




from django.conf import settings
from django.test import TestCase

from slumber import client
from slumber.connector.middleware import Cache

from slumber_examples.tests.client import TestsWithPizza
from slumber_examples.tests.configurations import ConfigureUser

from mock import patch


class _FailingMiddleware:
    def process_request(self, request):
        assert False, "This middleware is meant to fail."

class TestAddMiddleware(ConfigureUser, TestCase):
    def setUp(self):
        settings.MIDDLEWARE_CLASSES.append(
            'slumber_examples.tests.middleware._FailingMiddleware')
    def tearDown(self):
        settings.MIDDLEWARE_CLASSES.remove(
            'slumber_examples.tests.middleware._FailingMiddleware')

    def test_middleware_fails(self):
        with self.assertRaises(AssertionError):
            self.client.get('/')


class TestMiddleware(TestsWithPizza):
    def setUp(self):
        self.middleware = Cache()
        self.middleware.process_request(None)
        super(TestMiddleware, self).setUp()

    def tearDown(self):
        super(TestMiddleware, self).tearDown()
        self.middleware.process_response(None, None)


    def test_alias_writes_are_visible(self):
        m1 = client.slumber_examples.Pizza.get(pk=1)
        m2 = client.slumber_examples.Pizza.get(pk=1)
        self.assertEqual(m1.id, m2.id)
        with self.assertRaises(AttributeError):
            m1.attr
        with self.assertRaises(AttributeError):
            m2.attr
        m1.attr = 'attribute data'
        self.assertEqual(m1.attr, 'attribute data')
        self.assertEqual(m1.attr, m2.attr)


class TestSetting(ConfigureUser, TestCase):
    def test_request(self):
        called = []
        def flush_cache(*a):
            called.append(True)
        middleware = Cache()
        middleware.process_request(None)
        with patch('slumber.connector.Client._flush_client_instance_cache', flush_cache):
            response = middleware.process_response(None, 'response')
        self.assertTrue(called)
        self.assertEqual(response, 'response')

